# **********************************************************
# Copyright 2011-2015 VMware, Inc.  All rights reserved. -- VMware Confidential
# **********************************************************

from .VmomiSupport import GetVmodlType

class StubAdapterAccessorMixin:
   def __init__(self):
      self._pc = None
      self._pcType = GetVmodlType("vmodl.query.PropertyCollector")
      self._siType = GetVmodlType("vim.ServiceInstance")

   ## Retrieve a managed property
   #
   # @param self self
   # @param mo managed object
   # @param info property info
   def InvokeAccessor(self, mo, info):
      filterSpec = self._pcType.FilterSpec(
         objectSet=[self._pcType.ObjectSpec(obj=mo, skip=False)],
         propSet=[self._pcType.PropertySpec(all=False, type=mo.__class__,
                                                 pathSet=[info.name])],
         )
      ## Cache the property collector if it isn't already
      #  No need to lock _pc since multiple instances of PropertyCollector on
      #  the client will talk to the same instance on the server.
      if not self._pc:
         si = self._siType("ServiceInstance", self)
         self._pc = si.RetrieveContent().propertyCollector
      result = self._pc.RetrievePropertiesEx(
                  specSet=[filterSpec],
                  options=self._pcType.RetrieveOptions(maxObjects=1))
      objectContent = result.objects[0]
      if len(objectContent.propSet) > 0:
         return objectContent.propSet[0].val
      if len(objectContent.missingSet) > 0 and objectContent.missingSet[0].fault:
         raise objectContent.missingSet[0].fault
      return None
