#!/usr/bin/env python

from pyVim import connect, arguments
from pyVmomi import vim
from pyVim.helpers import Log
import os, sys

def main():
   supportedArgs = [ (["u:", "user="], "", "User name", "user"),
                     (["p:", "pwd="], "", "Password", "pwd"),
                     (["a:", "action="], "create", "[create|delete]", "action") ]

   supportedToggles = [ (["usage", "help"], False, "Show usage information", "usage"),]

   args = arguments.Arguments(sys.argv, supportedArgs, supportedToggles)
   if args.GetKeyValue("usage") == True:
      args.Usage()
      sys.exit(0)

   si = connect.Connect(host='localhost', user='root')

   authMgr = si.content.authorizationManager
   acctMgr = si.content.accountManager

   username = args.GetKeyValue("user")
   password = args.GetKeyValue("pwd")

   action = args.GetKeyValue("action")
   if action == "create":
      acctSpec = vim.host.LocalAccountManager.AccountSpecification()
      acctSpec.id = username
      acctSpec.password = password
      acctSpec.description = "Test user creation and login"
      acctMgr.CreateUser(acctSpec)
      Log("Created user %s" % username)

      perm = vim.AuthorizationManager.Permission()
      perm.group = False
      perm.principal = username
      perm.propagate = True
      perm.roleId = -1 # administrator
      authMgr.SetEntityPermissions(si.content.rootFolder, [perm])
      Log("Set permissions to Administrator for %s" % username)
   elif action == "delete":
      acctMgr.RemoveUser(username)
      Log("Removed user %s" % username)
   else:
      Log("Invalid action: '%s'" % action)
      connect.Disconnect()
      sys.exit(1)
   si.content.sessionManager.Logout()

if __name__ == "__main__":
   main()
