#!/usr/bin/python

### XXX: To be done
### - Unit test sets mac for nics
### but I dont know what a good static mac is so temporarily set wakeonlan
### - Unregister/register + all soft ops
### - Remove snapshot
### - util Vpxa compat
### - create vm with floppy image and cd image
### - lots more validations
### - remove long2to3() once migration to Python3 is complete

from __future__ import print_function

from optparse import OptionParser
import os
import sys
# hack: include six to support python3 for pyvmomi
sys.path.append('/build/toolchain/noarch/six-1.9.0/lib/python2.7/site-packages')
from pyVmomi import Vim, Vmodl, Hostd
from pyVim import folder, host, path, vimutil, vm, vmconfig, invt
from pyVim.connect import SmartConnect, Disconnect, GetSi
from pyVim.helpers import Log, StopWatch
from pyVim.task import WaitForTask
import time
import atexit
import re
import socket
import threading
import traceback

VM_NAME_PREFIX = os.environ.get('TESTESX_PREFIX', 'QaUnitTestVM')


def long2to3(x):
   """
   Migrating to Python3.
   @param x: Value to be returned as long or int based on Python version
   @return: int if version is equal to or greater than Python3, long otherwise
   """
   if sys.version_info[0] < 3:
      return long(x)
   else:
      return int(x)

isMockup = None

def IsMockup():
   global isMockup
   if isMockup is None:
      isMockup = host.IsHostMockup(GetSi())
   return isMockup

class MonitorProperties(threading.Thread):
   def __init__(self, si):
      self.si = si
      threading.Thread.__init__(self)

   def run(self):
      si = self.si

      pc = si.content.propertyCollector
      viewManager = si.content.viewManager

      containerView = viewManager.CreateContainerView(
         container=invt.GetVmFolder(),
         recursive=True)

      propSpec = Vmodl.query.PropertyCollector.PropertySpec(
         type=Vim.VirtualMachine,
         all=True)
      selectSpec = Vmodl.query.PropertyCollector.TraversalSpec(
         name='traverseView',
         path='view',
         skip=False,
         type=containerView.__class__)
      objSpec = Vmodl.query.PropertyCollector.ObjectSpec(
         obj=containerView,
         skip=False,
         selectSet=[selectSpec])
      spec = Vmodl.query.PropertyCollector.FilterSpec(
         propSet=[propSpec],
         objectSet=[objSpec])

      filter = pc.CreateFilter(spec, True)
      version = None
      while True:
         retrieveResult = pc.WaitForUpdates(version)
         if retrieveResult:
            version = retrieveResult.version

class Phase:
   def __init__(self):
      self.phaseNumber = 0
      self.phaseClock = StopWatch()

   def SetPhase(self, msg):
      Log("Phase " + str(self.phaseNumber) + ": " + msg + " completed")
      self.phaseClock.finish("phase " + str(self.phaseNumber))
      self.phaseClock = StopWatch()
      self.phaseNumber = self.phaseNumber + 1

def PowerCycle(vm1):
   Log("Powercycle (on/off)")
   vm.PowerOn(vm1)
   time.sleep(2)
   vm.PowerOff(vm1)
   time.sleep(2)

def UpdateVideoRamSize(vm1, video, vRamSize, isValid):
   cspec = Vim.Vm.ConfigSpec()
   video.SetVideoRamSizeInKB(long2to3(vRamSize))
   vmconfig.AddDeviceToSpec(cspec, video, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   try:
      task = vm1.Reconfigure(cspec)
      WaitForTask(task)
   except Exception as e:
      if isValid == False:
         Log("Verified rejection of invalid video ram size " + str(vRamSize))
         return
      else:
         raise
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualVideoCard,
				  {"videoRamSizeInKB": vRamSize})
   if len(devices) != 1:
      raise Exception("Failed to edit video ram size")
   Log("Verified successful edit of video ram size")
   PowerCycle(vm1)

def MessWithNpivWwn(vm1):
   if vm1.GetCapability().GetVmNpivWwnSupported() == False:
      Log("Not mess with NPIV WWN as it is not supported")
      return

   configSpec = Vim.vm.ConfigSpec()
   configSpec.npivWorldWideNameOp = "generate"
   WaitForTask(vm1.Reconfigure(configSpec))
   if len(vm1.GetConfig().GetNpivPortWorldWideName()) < 1:
       raise Exception("Failed to generate NPIV WWN")

   configSpec.npivWorldWideNameOp = "remove"
   WaitForTask(vm1.Reconfigure(configSpec))
   if len(vm1.GetConfig().GetNpivPortWorldWideName()) != 0:
       raise Exception("Failed to remove NPIV WWN")

   configSpec.npivDesiredNodeWwns = 2
   configSpec.npivDesiredPortWwns = 2
   configSpec.npivWorldWideNameOp = "generate"
   WaitForTask(vm1.Reconfigure(configSpec))
   npivPortWwn = vm1.GetConfig().GetNpivPortWorldWideName()
   npivNodeWwn = vm1.GetConfig().GetNpivNodeWorldWideName()
   if len(npivPortWwn) != 2:
       raise Exception("Failed to generate NPIV WWN desired")

   configSpec.npivWorldWideNameOp = "remove"
   WaitForTask(vm1.Reconfigure(configSpec))
   if len(vm1.GetConfig().GetNpivPortWorldWideName()) != 0:
       raise Exception("Failed to remove NPIV WWN")

   configSpec.npivPortWorldWideName = npivPortWwn
   configSpec.npivNodeWorldWideName = npivNodeWwn
   configSpec.npivTemporaryDisabled = False
   configSpec.npivWorldWideNameType = "vc"
   configSpec.npivWorldWideNameOp = "set"
   WaitForTask(vm1.Reconfigure(configSpec))
   if vm1.GetConfig().GetNpivPortWorldWideName() != npivPortWwn:
      raise Exception("Failed to set NPIV WWN Port")
   if vm1.GetConfig().GetNpivNodeWorldWideName() != npivNodeWwn:
      raise Exception("Failed to set NPIV WWN Node")
   if vm1.GetConfig().npivTemporaryDisabled != False:
      raise Exception("Failed to enable NPIV WWN Node")

   configSpec.npivDesiredNodeWwns = 3
   configSpec.npivDesiredPortWwns = 3
   configSpec.npivWorldWideNameOp = "extend"
   WaitForTask(vm1.Reconfigure(configSpec))
   if len(vm1.GetConfig().GetNpivPortWorldWideName()) != 3:
      raise Exception("Failed to extend NPIV WWN")

   configSpec.npivWorldWideNameOp = "remove"
   configSpec.npivNodeWorldWideName = None
   configSpec.npivPortWorldWideName = None
   WaitForTask(vm1.Reconfigure(configSpec))

def TryRemoteConsole(vm1):
   Log("Power on")
   vm.PowerOn(vm1)
   time.sleep(2)
   Log("Acquire ticket (mks)")
   vm1.AcquireTicket('mks')
   Log("Acquire ticket (device)")
   vm1.AcquireTicket('device')
   Log("Power off")
   vm.PowerOff(vm1)
   time.sleep(2)

def MessWithPowerOps(vm1):
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOff):
      raise Exception("Power state is not off.")
   Log("Power on")
   vm.PowerOn(vm1)
   time.sleep(2)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOn):
      raise Exception("Power state is not on.")
   Log("Reload")
   vm1.Reload()
   time.sleep(2)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOn):
      raise Exception("Power state is not on.")
   Log("Reset")
   vm.Reset(vm1)
   time.sleep(2)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOn):
      raise Exception("Power state is not on.")
   Log("Suspend")
   vm.Suspend(vm1)
   time.sleep(2)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.suspended):
      raise Exception("Power state is not suspended.")
   PowerCycle(vm1)
   PoweroffSuspendedVMTests(vm1)

def PoweroffSuspendedVMTests(vm1):
   Log("Poweroff Suspended VM")

   task = vm1.PowerOn()
   WaitForTask(task)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOn):
      raise Exception("Power state is not on.")
   task = vm1.Suspend()
   WaitForTask(task)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.suspended):
      raise Exception("Power state is not suspended.")
   # Now try powering off the VM
   task = vm1.PowerOff()
   WaitForTask(task)
   if (vm1.runtime.powerState != Vim.VirtualMachine.PowerState.poweredOff):
      raise Exception("Power state is not off.")
   Log("Successfully powered off Suspended VM")

def MessWithUuid(vm1):
   # Newly-created VM: UUID and Location ID should be non-empty and match
   if not vm1.config.uuid:
      raise Exception("UUID is empty")
   if vm1.config.uuid != vm1.config.locationId:
      raise Exception("UUID and Location ID don't match")

   # Reconfigure UUIDs with empty strings: clears Location ID, no-op for others
   (uuid, instanceUuid) = (vm1.config.uuid, vm1.config.instanceUuid)
   cspec = Vim.Vm.ConfigSpec(uuid='', instanceUuid='', locationId='')
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   if vm1.config.uuid != uuid:
      raise Exception("Empty UUID in ConfigSpec not ignored")
   if vm1.config.instanceUuid != instanceUuid:
      raise Exception("Empty Instance UUID in ConfigSpec not ignored")
   if vm1.config.locationId != '':
      raise Exception("Empty Location ID in ConfigSpec didn't clear the value")

   # Reconfigure UUID, verify that operation succeeded and event was generated
   newUuid = '01234567-89ab-cdef-fedc-ba9876543210'
   cspec = Vim.Vm.ConfigSpec(uuid=newUuid)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   if newUuid != vm1.config.uuid:
      raise Exception("UUID not updated correctly: expected %s, found %s" %
                      (newUuid, vm1.config.uuid))
   if vm1.config.locationId != '':
      raise Exception("Empty Location ID in ConfigSpec didn't clear the value")
   eventManager = GetSi().RetrieveContent().GetEventManager()
   eventType = "VmUuidChangedEvent"
   espec = Vim.Event.EventFilterSpec()
   espec.eventTypeId = [eventType]
   espec.entity = Vim.Event.EventFilterSpec.ByEntity(
      entity=vm1, recursion=Vim.Event.EventFilterSpec.RecursionOption.self)
   collector = eventManager.CreateCollector(espec)
   eventList = collector.latestPage
   if len(eventList) == 0:
      raise Exception("No %s event generated after UUID change" % eventType)
   elif len(eventList) > 1:
      raise Exception("Multiple %s events generated for single UUID change" % eventType)
   event = eventList[0]
   if uuid != event.oldUuid:
      raise Exception("Incorrect oldUuid reported in %s: expected %s, found %s" %
                      (eventType, uuid, event.oldUuid))
   if newUuid != event.newUuid:
      raise Exception("Incorrect newUuid reported in %s: expected %s, found %s" %
                      (eventType, newUuid, event.newUuid))

def MessWithSata(vm1, datastore=None):
   global about
   if about.productLineId == 'ws':
      Log("Skipping sata reconfigure for WS")
      return

   # Add virtual disk to VM
   Log("Add a new SATA disk to the VM.")
   diskName = vm1.GetConfig().GetName() + ".vmdk"
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddSataCtlr(cspec)
   vmconfig.AddDisk(cspec, "sata", datastorename=datastore, fileName=diskName,
                    createNamedFile=True)
   vm.Reconfigure(vm1, cspec)
   ctlrs = vmconfig.CheckDevice(vm1.config, Vim.Vm.Device.VirtualSATAController)

   if len(ctlrs) != 1:
      raise Exception("Failed to find added SATA controller!")

   devices = vmconfig.CheckDevice(vm1.config, Vim.Vm.Device.VirtualDisk,
                                  {"controllerKey": ctlrs[0].key})
   if len(devices) < 1:
      raise Exception("Failed to find SATA disk!")

   disk = devices[0]
   Log("Reconfigure disk capacity.")
   cspec = Vim.Vm.ConfigSpec()
   newCapacity = disk.capacityInKB + 16384
   disk.capacityInKB = newCapacity
   vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   vm.Reconfigure(vm1, cspec)
   devices = vmconfig.CheckDevice(vm1.config, Vim.Vm.Device.VirtualDisk,
                                  {"capacityInKB": newCapacity, "key": disk.key})
   if len(devices) != 1:
      raise Exception("Failed to find the disk with new capacity.")

   Log("Hot removing SATA disk.")
   vm.PowerOn(vm1)
   cspec = Vim.Vm.ConfigSpec()
   fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
   vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
   vm.Reconfigure(vm1, cspec)
   vm.PowerOff(vm1)

   Log("Adding a new SATA cdrom to the VM.")
   cspec = Vim.Vm.ConfigSpec()
   try:
     vmconfig.AddSataCdrom(cspec, cfgInfo = vm1.config)
   except Exception as e:
      if str(e) == "No device available on the host to add":
         Log("No cdrom drives available, skipping SATA cdrom tests")
         vm.RemoveDevice(vm1, ctlrs[0])
         return
      else:
         raise
   vm.Reconfigure(vm1, cspec)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualCdrom)
   if len(devices) < 1:
      raise Exception("Failed to find SATA cdrom!")

   cdrom = devices[0]
   Log("Reconfigure cdrom backing.")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualCdrom.IsoBackingInfo(fileName="[]")
   cdrom.backing = backing
   vmconfig.AddDeviceToSpec(cspec, cdrom, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   vm.Reconfigure(vm1, cspec)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualCdrom,
                                  {"backing.fileName": "[]"})
   if len(devices) != 1:
      raise Exception("Failed to find edited cdrom!")

   # Remove SATA devices and controller from VM
   Log("Removing disk and cdrom from the VM")
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.RemoveDeviceFromSpec(cspec, cdrom)
   vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
   vm.Reconfigure(vm1, cspec)

def MessWithDisks(vm1, newName, datastore=None):
   # Add virtual disk to VM
   Log("Add a new disk to the VM")
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiCtlr(cspec)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
   if len(devices) < 2:
      raise Exception("Failed to find added disk!")
   disk = devices[1]
   ctlrKey = disk.GetControllerKey()
   ctlrs = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSCSIController,
				  {"key": ctlrKey})
   if len(ctlrs) != 1:
      raise Exception("Failed to find added SCSI controller!")

   # Small verification of uuid
   if not disk.GetBacking().GetUuid():
      raise Exception("Disk backing has no UUID")
   uuid = "60 00 C2 9f 58 02 9a 2a-90 dc 94 52 18 88 1a 5d"
   shortUuid=uuid.replace(' ', '').replace('-', '')

   diskFile = disk.GetBacking().GetFileName()
   si = GetSi()
   vDiskManager = si.GetContent().GetVirtualDiskManager()
   if vDiskManager: # hosted doesn't have one
      vDiskManager.SetVirtualDiskUuid(diskFile, uuid=uuid)
      vm1.Reload()
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
      if len(devices) < 2:
         raise Exception("Failed to find added disk!")
      disk = devices[1]
      currentUuid = disk.GetBacking().GetUuid().replace(' ', '').replace('-', '')
      if currentUuid != shortUuid:
         raise Exception("VM has unexpected uuid %s != %s" %
                         (currentUuid, shortUuid))

   if not IsMockup():
      MessWithIndependentDiskSnapshot(vm1, datastore)

   # Try to hot add a new disk with a parent in a different directory
   vm.CreateSnapshot(vm1, "asParent", "asParent", False, False)
   s1 = vm1.snapshot.currentSnapshot
   parentDisk = vmconfig.CheckDevice(s1.config, Vim.Vm.Device.VirtualDisk)[1]
   parentDiskPath = parentDisk.backing.fileName
   vmxPath = vm1.summary.config.vmPathName
   if datastore is None:
      m = re.search(r'\[(.+)\]', vmxPath)
      datastore = m.group(1)
   vmName = newName + 'disk'
   vm2 = vm.CreateQuickDummy(vmName, 1, datastoreName=datastore)
   vmDir = os.path.dirname(vm2.summary.config.vmPathName)
   m = re.search(r'\[.+\] *(.+)', vmDir)
   vmDir = m.group(1)
   Log("Test hot-add of a new disk '%s' with parent '%s'" %
       (vmDir + '/newDisk.vmdk', parentDiskPath))
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiDisk(cspec, fileName = vmDir + '/newDisk.vmdk',
                                datastorename = datastore,
                                diskmode = 'independent_persistent',
                                thin = True, createNamedFile = True,
                                cfgInfo = vm2.config)
   diskBacking = cspec.deviceChange[0].device.backing
   parentBacking = Vim.Vm.Device.VirtualDisk.FlatVer2BackingInfo()
   parentBacking.fileName = parentDiskPath
   diskBacking.parent = parentBacking
   vm.PowerOn(vm2)
   WaitForTask(vm2.Reconfigure(cspec))
   devices = vmconfig.CheckDevice(vm2.config, Vim.Vm.Device.VirtualDisk)
   if len(devices) < 2:
      raise Exception("Failed to find added disk!")
   vm.PowerOff(vm2)
   # Verify VM is still good and clean up
   PowerCycle(vm2)
   vm.Destroy(vm2)
   vm.RemoveAllSnapshots(vm1)

   # Remove virtual disk and controller from VM
   Log("Removing disk and controller from the VM")
   cspec = Vim.Vm.ConfigSpec()
   fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
   vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
   vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   # Try to hot add a disk in a different datastore
   datastores = vm1.GetDatastore()
   hostSystem = host.GetHostSystem(si)
   ds2 = None
   for ds in hostSystem.datastore:
      if ds.summary.type == 'VMFS' and ds not in datastores:
         ds2 = ds
         break
   if ds2:
      Log("Hot-add a new disk on datastore %s to the VM" % ds2.summary.name)
      cspec = Vim.Vm.ConfigSpec()
      cspec = vmconfig.AddScsiCtlr(cspec)
      vmconfig.AddScsiDisk(cspec, datastorename=ds2.summary.name)
      vm.PowerOn(vm1)
      WaitForTask(vm1.Reconfigure(cspec))
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
      if len(devices) < 2:
         raise Exception("Failed to find added disk!")
      disk = devices[1]
      ctlrKey = disk.GetControllerKey()
      ctlrs = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSCSIController,
                                     {"key": ctlrKey})
      if len(ctlrs) != 1:
         raise Exception("Failed to find added SCSI controller!")
      vm.PowerOff(vm1)
      Log("Removing hot-added disk and controller from the VM")
      cspec = Vim.Vm.ConfigSpec()
      fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
      vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
      vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
      WaitForTask(vm1.Reconfigure(cspec))

   if not IsMockup():

      # increase disk size
      Log("Increase disk size by 8 MB")
      #raw_input("-->")
      cspec = Vim.Vm.ConfigSpec()
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
      if len(devices) < 1:
         raise Exception("Failed to find a suitable disk")

      disk = devices[0]
      cspec = Vim.Vm.ConfigSpec()
      newCapacity = disk.GetCapacityInKB() + 8192
      disk.SetCapacityInKB(newCapacity)
      vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      WaitForTask(task)
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk,
                                     {"capacityInKB": newCapacity})
      if len(devices) != 1:
         raise Exception("Failed to find the disk again")

      # try to set capacityInKB to 0
      Log("Set capacityInKB to 0")
      disk = vmconfig.CheckDevice(vm1.config, Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInKB = 0
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInKB to 0 as expected")
      else:
         raise Exception("Error: Setting capacityInKB to 0 was allowed!")

      # try to set capacityInKB to -1
      Log("Set capacityInKB to -1")
      disk = vmconfig.CheckDevice(vm1.config, Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInKB = -1
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInKB to -1 as expected")
      else:
         raise Exception("Error: Setting capacityInKB to -1 was allowed!")

      # try to set capacityInBytes to 0
      Log("Set capacityInBytes to 0")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInBytes = 0
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInBytes to 0 as expected")
      else:
         raise Exception("Error: Setting capacityInBytes to 0 of disk was allowed!")

      # try to set capacityInBytes to -1
      Log("Set capacityInBytes to -1")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInBytes = -1
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInBytes to -1 as expected")
      else:
         raise Exception("Error: Setting capacityInBytes to -1 of disk was allowed!")

      # try to set both capacityInKB and capacityInBytes to 0
      Log("Set capacityInKB and capacityInBytes to 0")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInKB = 0
      disk.capacityInBytes = 0
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInKB and capacityInBytes to 0 as expected")
      else:
         raise Exception("Error: Setting capacityInKB and capacityInBytes to 0 of disk was allowed!")

      # try to set both capacityInKB and capacityInBytes to -1
      Log("Set capacityInKB and capacityInBytes to -1")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.capacityInKB = -1
      disk.capacityInBytes = -1
      vmconfig.AddDeviceToSpec(
         cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when setting capacityInKB and capacityInBytes to -1 as expected")
      else:
         raise Exception("Error: Setting capacityInKB and capacityInBytes to -1 of disk was allowed!")

      # try to decrease disk size
      Log("Decrease disk size")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      newCapacity = disk.GetCapacityInKB() - 4096
      disk.SetCapacityInKB(newCapacity)
      vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception when decreasing size as expected" + str(e))
      else:
         raise Exception("Error: Decreasing size of disk was allowed!")

      # attempt to extend a disk with a parent
      Log("Creating a snapshot on the VM for negative disk extend test")
      vm.CreateSnapshot(vm1, "dummy snapshot", "dummy desc", False, False)
      Log("Attempting to extend disk size of delta disk")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      disk.SetCapacityInKB(disk.GetCapacityInKB() + 4096)
      vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      try:
         WaitForTask(task)
      except Exception as e:
         Log("Hit an exception extending delta disk as expected" + str(e))
      else:
         raise Exception("Error: Extending delta disk was allowed!")

      # add a disk in a different directory, take a snapshot then delete disk
      # power on the vm and delete the snapshot (which should remove the disk)
      Log("Adding a disk outside of the VM directory for snapshot test")
      pathBase = os.path.dirname(vm1.GetConfig().GetFiles().GetVmPathName())
      diskDir = pathBase + "-disks"
      fileManager = GetSi().RetrieveContent().GetFileManager()
      try:
         fileManager.MakeDirectory(diskDir)
      except Vim.Fault.FileAlreadyExists as e:
         pass
      cspec = Vim.Vm.ConfigSpec()
      cspec = vmconfig.AddScsiCtlr(cspec)
      diskFullPath = diskDir + "/extDisk.vmdk"
      diskPath = re.sub(r'\[.*\]\s*', '', diskFullPath)
      vmconfig.AddScsiDisk(cspec, datastorename=datastore, fileName=diskPath,
                           createNamedFile=True)
      vm.Reconfigure(vm1, cspec)
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
      if len(devices) != 2:
         raise Exception("Failed to find added external disk!")
      disk = devices[1]
      ctlrKey = disk.GetControllerKey()
      ctlrs = vmconfig.CheckDevice(vm1.GetConfig(),
                                   Vim.Vm.Device.VirtualSCSIController,
				  {"key": ctlrKey})
      if len(ctlrs) != 1:
         raise Exception("Failed to find added SCSI controller!")
      vm.PowerOn(vm1)
      vm.CreateSnapshot(vm1, "dummy snapshot 2", "dummy desc", True, False)
      ss2 = vm1.snapshot.currentSnapshot
      vm.PowerOff(vm1)
      cspec = Vim.Vm.ConfigSpec()
      fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
      vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
      vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
      vm.Reconfigure(vm1, cspec)
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
      if len(devices) != 1:
         raise Exception("Failed to remove disk and controller")
      vm.PowerOn(vm1)
      vm.RemoveSnapshot(ss2)
      vm.PowerOff(vm1)
      datastores = vm1.GetDatastore()
      if DirectoryExists(datastores, diskFullPath):
         raise Exception("Disk file has been left in %s" % diskDir)
      vimutil.InvokeAndTrack(fileManager.Delete, datastorePath=diskDir,
                             fileType=Vim.FileManager.FileType.File)

      Log("Removing all snapshots on the VM")
      vm.RemoveAllSnapshots(vm1)

      Log("Removing all snapshots on the snapshot-less VM")
      vm.RemoveAllSnapshots(vm1)

      # Change the disk uuid
      Log("Change disk uuid")
      disk = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)[0]
      cspec = Vim.Vm.ConfigSpec()
      backing = disk.GetBacking()
      uuid = "6000C29f-c2a6-5c04-3bab-aa781ddeff37"
      backing.SetUuid(uuid)
      disk.SetBacking(backing)
      vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
      task = vm1.Reconfigure(cspec)
      WaitForTask(task)
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk,
                                     {"backing.uuid": uuid})
      if len(devices) != 1:
         raise Exception("Failed to set disk uuid")


   # Try to power on the vm and then extend the disk
   #Log("Poweron and increase disk size")
   #disk = devices[0]
   #vm.PowerOn(vm1)
   #cspec = Vim.Vm.ConfigSpec()
   #newCapacity = disk.GetCapacityInKB() + 8192
   #disk.SetCapacityInKB(newCapacity)
   #vmconfig.AddDeviceToSpec(cspec, disk, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   #task = vm1.Reconfigure(cspec)
   #WaitForTask(task)
   #devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk,
   #                               {"capacityInKB": newCapacity})
   #if len(devices) != 1:
   #   raise Exception("Failed to find the disk again")
   #vm.PowerOff(vm1)

def MessWithIndependentDiskSnapshot(vm1, datastore=None):
   # Add an independent virtual disk to VM
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiCtlr(cspec)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore, diskmode = "independent_persistent")
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
   if len(devices) < 3:
      raise Exception("Failed to find added disk!")
   disk = devices[1]
   independent_disk = devices[2]

   # Test removal of vdisk which is part of a VM snapshot
   Log("Creating VM Snapshot with new disks added.")
   vm.CreateSnapshot(vm1, "dummy snapshot", "dummy desc", False, False)

   Log("Removing non-independent disk from VM after Snapshot.")
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.RemoveDeviceFromSpec(cspec, disk)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Exception as e:
      Log("Hit an expected exception while removing disk which is part of a VM snapshot." + str(e))
   else:
      raise Exception("Error: Removing snapshotted disk was allowed.")

   Log("Removing independent disk from VM after Snapshot.")
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.RemoveDeviceFromSpec(cspec, independent_disk)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Exception as e:
      Log("Hit an expected exception while removing disk which is part of a VM snapshot." + str(e))
   else:
      raise Exception("Error: Removing independent disk which is part of a VM snapshot was allowed.")

   Log("Removing VM Snapshot.")
   vm.RemoveAllSnapshots(vm1)

   cspec = Vim.Vm.ConfigSpec()
   fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
   vmconfig.RemoveDeviceFromSpec(cspec, independent_disk, fileOp)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)


def MessWithReconfigMisc(vm1):
   global about
   if about.productLineId == 'ws':
      Log("Skipping reconfigure misc for WS")
      return

   cspec = Vim.Vm.ConfigSpec()
   flags = Vim.Vm.FlagInfo()
   cspec.SetFlags(flags)

   for logging in [False, True]:
      flags.SetEnableLogging(logging)
      WaitForTask(vm1.Reconfigure(cspec))
      currLogging = vm1.GetConfig().GetFlags().GetEnableLogging()
      if logging != currLogging:
         raise Exception("Failed to update enableLogging correctly: %s != %s"
                         % (logging, currLogging))
      vm1.Reload()
      currLogging = vm1.GetConfig().GetFlags().GetEnableLogging()
      if logging != currLogging:
         raise Exception("enableLogging not correct after reload: %s != %s"
                         % (logging, currLogging))

   origDiskUuid = vm1.GetConfig().GetFlags().GetDiskUuidEnabled()
   uuidSettings = [True, False, origDiskUuid]
   for uuid in uuidSettings:
      flags.SetDiskUuidEnabled(uuid)
      WaitForTask(vm1.Reconfigure(cspec))
      currUuidSetting = vm1.GetConfig().GetFlags().GetDiskUuidEnabled()
      if uuid != currUuidSetting:
         raise Exception("Failed to update diskUuidEnabled correctly: %s != %s"
                         % (uuid, currUuidSetting))
      vm1.Reload()
      currUuidSetting = vm1.GetConfig().GetFlags().GetDiskUuidEnabled()
      if uuid != currUuidSetting:
         raise Exception("diskUuidEnabled not correct after reload: %s != %s"
                         % (uuid, currUuidSetting))
   flags.SetDiskUuidEnabled(None)

   htSettings = [Vim.vm.FlagInfo.HtSharing.none,
                 Vim.vm.FlagInfo.HtSharing.internal,
                 Vim.vm.FlagInfo.HtSharing.any]

   for ht in htSettings:
      flags.SetHtSharing(ht)
      WaitForTask(vm1.Reconfigure(cspec))
      currHt = vm1.GetConfig().GetFlags().GetHtSharing()
      if ht != currHt:
         raise Exception("Failed to update htSharing correctly: %s != %s"
                         % (ht, currHt))
      vm1.Reload()
      currHt = vm1.GetConfig().GetFlags().GetHtSharing()
      if ht != currHt:
         raise Exception("htSharing not correct after reload: %s != %s"
                         % (ht, currHt))

   flags = Vim.Vm.FlagInfo()
   cspec.SetFlags(flags)
   monitorSettings = [Vim.vm.FlagInfo.MonitorType.stats,
                      Vim.vm.FlagInfo.MonitorType.debug,
                      Vim.vm.FlagInfo.MonitorType.release]

   extraCfg = [Vim.option.OptionValue(key='vui.timeoutUS.post', value=1),
               Vim.option.OptionValue(key='vui.timeoutUS.hint', value=1)]
   WaitForTask(vm1.Reconfigure(Vim.vm.ConfigSpec(extraConfig=extraCfg)))

   for monitor in monitorSettings:
      # Alternate between hot-update and regular update
      if vm1.GetRuntime().GetPowerState() == Vim.VirtualMachine.PowerState.poweredOn:
         vm.PowerOff(vm1)
      else:
         vm.PowerOn(vm1)

      flags.SetMonitorType(monitor)
      WaitForTask(vm1.Reconfigure(cspec))
      currMonitor = vm1.GetConfig().GetFlags().GetMonitorType()
      if monitor != currMonitor:
         raise Exception("Failed to update monitorType correctly: %s != %s"
                         % (monitor, currMonitor))
      vm1.Reload()
      currMonitor = vm1.GetConfig().GetFlags().GetMonitorType()
      if monitor != currMonitor:
         raise Exception("monitorType not correct after reload: %s != %s"
                         % (monitor, currMonitor))

   if vm1.GetRuntime().GetPowerState() == Vim.VirtualMachine.PowerState.poweredOn:
      vm.PowerOff(vm1)
   extraCfg = [Vim.option.OptionValue(key='vui.timeoutUS.post', value=""),
               Vim.option.OptionValue(key='vui.timeoutUS.hint', value="")]
   WaitForTask(vm1.Reconfigure(Vim.vm.ConfigSpec(extraConfig=extraCfg)))


def MessWithReconfigMonitorUsage(vm1):
   cspec = Vim.Vm.ConfigSpec()
   flags = Vim.Vm.FlagInfo()
   cspec.SetFlags(flags)

   mmuUsage = [Vim.Vm.FlagInfo.VirtualMmuUsage.on,
               Vim.Vm.FlagInfo.VirtualMmuUsage.off,
               Vim.Vm.FlagInfo.VirtualMmuUsage.automatic]
   execUsage = [Vim.Vm.FlagInfo.VirtualExecUsage.hvOn,
                Vim.Vm.FlagInfo.VirtualExecUsage.hvOff,
                Vim.Vm.FlagInfo.VirtualExecUsage.hvAuto]

   if (vm1.GetConfig().GetFlags().GetVirtualMmuUsage()
       != Vim.Vm.FlagInfo.VirtualMmuUsage.automatic):
      raise Exception("Default setting for virtual mmu usage is not automatic!")
   if (vm1.GetConfig().GetFlags().GetVirtualExecUsage()
       != Vim.Vm.FlagInfo.VirtualExecUsage.hvAuto):
      raise Exception("Default setting for virtual exec usage is not automatic!")

   for vMmu in mmuUsage:
      flags.SetVirtualMmuUsage(vMmu)
      WaitForTask(vm1.Reconfigure(cspec))
      currentMmu = vm1.GetConfig().GetFlags().GetVirtualMmuUsage()
      if currentMmu != vMmu:
         raise Exception("Setting for virtual mmu usage is not switch to %s (%s)" % (vMmu, currentMmu))
   flags.SetVirtualMmuUsage(None)

   for vExec in execUsage:
      flags.SetVirtualExecUsage(vExec)
      WaitForTask(vm1.Reconfigure(cspec))
      currentExec = vm1.GetConfig().GetFlags().GetVirtualExecUsage()
      if currentExec != vExec:
         raise Exception("Setting for virtual exec usage is not switch to %s (%s)" % (vExec, currentExec))
   flags.SetVirtualExecUsage(None)


def MessWithCPUVirtualization(vm1):
   def _getVal(vm, key):
      if key in vm.config.__dict__:
         return vm.config.__dict__.get(key)
      elif key in vm.config.hardware.__dict__:
         return vm.config.hardware.__dict__.get(key)
      else:
         raise Exception("Key %s not found" % key)

   ConfigSpec = Vim.Vm.ConfigSpec
   cspecs = [(True,  'virtualICH7MPresent', ConfigSpec(virtualICH7MPresent=True)),
             (False, 'virtualICH7MPresent', ConfigSpec(virtualICH7MPresent=False)),
             (True,  'virtualSMCPresent',   ConfigSpec(virtualSMCPresent=True)),
             (False, 'virtualSMCPresent',   ConfigSpec(virtualSMCPresent=False)),
             (True,  'vPMCEnabled',         ConfigSpec(vPMCEnabled=True)),
             (False, 'vPMCEnabled',         ConfigSpec(vPMCEnabled=False)),
             (True,  'nestedHVEnabled',     ConfigSpec(nestedHVEnabled=True)),
             (False, 'nestedHVEnabled',     ConfigSpec(nestedHVEnabled=False)),
            ]
   for expectedVal, key, spec in cspecs:
      WaitForTask(vm1.Reconfigure(spec))
      val = _getVal(vm1, key)
      if val != expectedVal:
         raise Exception("%s not expected value after reconfig %s != %s"
                         % (key, val, expectedVal))
      vm1.Reload()
      val = _getVal(vm1, key)
      if val != expectedVal:
         raise Exception("%s not expected value after reload %s != %s"
                         % (key, val, expectedVal))


def MessWithReconfigTools(vm1):
   cspec = Vim.Vm.ConfigSpec()
   tools = Vim.Vm.ToolsConfigInfo()
   cspec.SetTools(tools)

   for value in [False, True]:
      cspec.SetGuestAutoLockEnabled(value)
      cspec.SetMessageBusTunnelEnabled(value)
      tools.SetAfterPowerOn(value)
      tools.SetAfterResume(value)
      tools.SetBeforeGuestStandby(value)
      tools.SetBeforeGuestShutdown(value)
      tools.SetSyncTimeWithHost(value)
      WaitForTask(vm1.Reconfigure(cspec))
      for reloaded in ["", " after reload"]:
         curValue = vm1.GetConfig().GetGuestAutoLockEnabled()
         if curValue != value:
            raise Exception("Failed to set guestAutoLock correctly" + reloaded +
                            ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetMessageBusTunnelEnabled()
         if curValue != value:
            raise Exception("Failed to set messageBusTunnelEnabled correctly"
                            + reloaded + ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetTools().GetAfterPowerOn()
         if curValue != value:
            raise Exception("Failed to set afterPowerOn correctly" + reloaded +
                            ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetTools().GetAfterResume()
         if curValue != value:
            raise Exception("Failed to set afterResume correctly" + reloaded +
                            ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetTools().GetBeforeGuestStandby()
         if curValue != value:
            raise Exception("Failed to set beforeGuestStandby correctly" +
                            reloaded + ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetTools().GetBeforeGuestShutdown()
         if curValue != value:
            raise Exception("Failed to set beforeGuestShutdown correctly" +
                            reloaded + ": %s != %s" % (curValue, value))
         curValue = vm1.GetConfig().GetTools().GetSyncTimeWithHost()
         if curValue != value:
            raise Exception("Failed to set syncTimeWithHost correctly" +
                            reloaded + ": %s != %s" % (curValue, value))
         vm1.Reload()

   # Until the work-around for PR 953140 is removed, this does not work
   # with a mockup when VigorVim is enabled
   if not IsMockup():
      for policy in [Vim.vm.ToolsConfigInfo.UpgradePolicy.upgradeAtPowerCycle,
                     Vim.vm.ToolsConfigInfo.UpgradePolicy.manual]:
         tools.SetToolsUpgradePolicy(policy)
         WaitForTask(vm1.Reconfigure(cspec))
         for reloaded in ["", " after reload"]:
            curPolicy = vm1.GetConfig().GetTools().GetToolsUpgradePolicy()
            if curPolicy != policy:
               raise Exception("Failed to set upgradePolicy correctly" + reloaded +
                               ": %s != %s" % (curPolicy, policy))
            vm1.Reload()

   for package in ["packageName", ""]:
      tools.SetPendingCustomization(package)
      WaitForTask(vm1.Reconfigure(cspec))
      for reloaded in ["", " after reload"]:
         curPackage = vm1.GetConfig().GetTools().GetPendingCustomization()
         # We cannot use os.path.basename as that does not work when the local
         # platform (where python runs) and the remote platform (where hostd
         # runs) differ in their path name formats.
         basename = ''
         if curPackage != None:
            basename = re.split(r'[\\\/]', curPackage)[-1]
         if ((curPackage == None and package != "") or
             (curPackage != None and basename != package)):
            raise Exception("Failed to set pendingPackage correctly" +
                            reloaded + ": %s != %s" % (basename, package))
         vm1.Reload()

   vm.PowerOn(vm1)
   cspec = Vim.Vm.ConfigSpec()
   tools = Vim.Vm.ToolsConfigInfo()
   cspec.SetTools(tools)
   for value in [True, False]:
      tools.SetSyncTimeWithHost(value)
      WaitForTask(vm1.Reconfigure(cspec))
      curValue = vm1.GetConfig().GetTools().GetSyncTimeWithHost()
      if curValue != value:
         raise Exception("Failed to hot update syncTimeWithHost correctly: " +
                         "%s != %s" % (curValue, value))
   vm.PowerOff(vm1)


def MessWithAudio(vm1):
   # Check if there is a sound card available on the host
   if len(vmconfig.GetCfgTarget(None).GetSound()) == 0:
      Log("No sound card available on the host. Skipping sound card tests...")
      return

   # Add audio card
   Log("Add sound card")
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddSoundCard(cspec, autodetect = True)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSoundCard)
   if len(devices) != 1:
      raise Exception("Failed to find added sound card")
   sound = devices[0]

   Log("Remove sound card")
   vm.RemoveDevice(vm1, sound)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSoundCard)
   if len(devices) != 0:
      raise Exception("Found sound card even after delete")

def MessWithCdroms(vm1):
   # Add cdrom
   Log("Add cdrom")
   cspec = Vim.Vm.ConfigSpec()
   try:
     vmconfig.AddCdrom(cspec)
   except Exception as e:
      if str(e) == "No device available on the host to add":
         Log("No cdrom drives available, skipping cdrom tests")
         return
      else:
         raise
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualCdrom)
   if len(devices) != 1:
      raise Exception("Failed to find added cdrom")
   cdrom = devices[0]
   PowerCycle(vm1)

   # Reconfig cdrom to point to an iso backing
   Log("Reconfig cdrom to iso backing")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualCdrom.IsoBackingInfo()
   backing.SetFileName("[]")
   cdrom.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec, cdrom, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualCdrom,
                                  {"backing.fileName": "[]"})
   if len(devices) != 1:
      raise Exception("Failed to find edited cdrom")
   PowerCycle(vm1)

   # Remove cdrom
   Log("Remove cdrom")
   vm.RemoveDevice(vm1, devices[0])
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualCdrom)
   if len(devices) != 0:
      raise Exception("Found cdrom even after delete")
   PowerCycle(vm1)

def MessWithParallelPorts(vm1):
   Log("Add parallel port")
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualParallelPort)
   if len(devices) != 0:
      Log("A parallel port is already present: %s" % str(devices))
      raise Exception("Parallel port already exists")

   #Add a parallel port
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.AddFileBackedParallel(cspec, "[]")
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   #Check that the parallel port was added.
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualParallelPort)
   if len(devices) != 1:
      raise Exception("Failed to find added parallel port")
   parallelDev = devices[0]

   #Reconfigure the parallel port to a device backing
   Log("Reconfigure parallel port with device backing")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualParallelPort.DeviceBackingInfo()
   parallelDev.SetBacking(backing)
   backing.SetUseAutoDetect(True)
   vmconfig.AddDeviceToSpec(cspec,
                            parallelDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   Log("Try to reconfigure parallel port with invalid backing")
   cspec = Vim.Vm.ConfigSpec()
   #any backing of a device different than parallel could have been used
   backing = Vim.Vm.Device.VirtualSerialPort.PipeBackingInfo()
   parallelDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec, parallelDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure parallel port with invalid backing")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring parallel port with invalid backing")

   #Reconfigure the parallel port with a file backing, with invalid file name.
   #This corresponds to vmops/reconfigvm/Neg040.java test from vcqe.
   Log("Try to reconfigure parallel port with invalid file backing")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualParallelPort.FileBackingInfo()
   backing.SetFileName("xxx")
   parallelDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec,
                            parallelDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDatastore as e:
      #on VC InvalidDataStore path is expected
      Log("Hit an expected exception when trying to configure parallel port with invalid file")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring parallel port with invalid file")

   #Reconfigure the parallel port with backing set to NULL/None.
   #This corresponds to vmops/reconfigvm/Neg051.java test from vcqe.
   Log("Reconfigure parallel port with NULL backing")
   cspec = Vim.Vm.ConfigSpec()
   parallelDev.SetBacking(None)
   vmconfig.AddDeviceToSpec(cspec,
                            parallelDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
      #this will work only for ESXi 4.1+. This is not a problem.
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure parallel port with NULL backing")
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure parallel port with NULL backing")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring parallel port with with NULL backing")

   # Remove parallel
   Log("Remove parallel port")
   vm.RemoveDevice(vm1, devices[0])
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualParallelPort)
   if len(devices) != 0:
      raise Exception("Found parallel port even after delete")
   PowerCycle(vm1)


def MessWithSerialPorts(vm1):
   Log("Add serial port")
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSerialPort)
   if len(devices) != 0:
      Log("A serial port is already present: %s" % str(devices))
      raise Exception("Serial port already exists")

   #Add a serial port
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.AddFileBackedSerial(cspec, "[]")
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   #Check that the serial port was added.
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSerialPort)
   if len(devices) != 1:
      raise Exception("Failed to find added serial port")
   serialDev = devices[0]

   #Reconfigure the serial port to a device backing
   Log("Reconfigure serial port with device backing")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualSerialPort.DeviceBackingInfo()
   serialDev.SetBacking(backing)
   backing.SetUseAutoDetect(True)
   vmconfig.AddDeviceToSpec(cspec,
                            serialDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   #Reconfigure the serial port with a pipe backing, with wrong endpoint.
   #This corresponds to vmops/reconfigvm/Neg046.java test from vcqe.
   Log("Reconfigure serial port with pipe backing, invalid endpoint")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualSerialPort.PipeBackingInfo()
   backing.SetPipeName("/tmp/serialport")
   backing.SetEndpoint("xxx")
   serialDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec,
                            serialDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
      #this will work only for ESXi 4.1+. This is not a problem.
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure serial port with invalid pipe backing endpoint")
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure serial port with invalid pipe backing endpoint")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring serial port with with invalid pipe backing endpoint")

   #Reconfigure the serial port with backing set to NULL/None.
   #This corresponds to vmops/reconfigvm/Neg053.java test from vcqe.
   Log("Reconfigure serial port NULL backing")
   cspec = Vim.Vm.ConfigSpec()
   serialDev.SetBacking(None)
   vmconfig.AddDeviceToSpec(cspec,
                            serialDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
      #this will work only for ESXi 4.1+. This is not a problem.
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure serial port with NULL backing")
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure serial port with NULL backing")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring serial port with with NULL backing")

   #Reconfigure serial port with network backing, with Proxy URI set to empty string.
   #This reproduces PR929623
   Log("Reconfiguring serial port with network backing, empty proxy URI")
   backing = Vim.Vm.Device.VirtualSerialPort.URIBackingInfo()
   backing.SetServiceURI("telnet://10.117.9.127:3000")
   backing.SetDirection("client")
   backing.SetProxyURI("")
   serialDev.SetBacking(backing)
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.AddDeviceToSpec(cspec,
                            serialDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Exception:
      Log("Hit an unexpected exception when trying to reconfigre serial port with empty proxy URI")
      raise Exception("Hit an unexpected exception when trying to reconfigre serial port with empty proxy URI")

   # Remove serial
   Log("Remove serial port")
   vm.RemoveDevice(vm1, devices[0])
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualSerialPort)
   if len(devices) != 0:
      raise Exception("Found serial port even after delete")
   PowerCycle(vm1)

def MessWithFloppies(vm1):
   #
   # The fileName specified below isn't in an exposed path on WS for Shared VMs.
   # No alternate flp files are found on the Shared VMs datastore by default.
   # So use something faked on WS.
   #
   # Need to use a file path that passes the config_rules checks.
   # Also, using a dummy dsPath like "[]" causes this test to
   # fail against obj hostd builds because of platform defaults
   # for floppy backings reverting to /dev/fd0.
   #
   global about
   if about.productLineId != 'ws':
      image = "[] /vmimages/floppies/vmscsi.flp"
   else:
      # Path to floppy needs to be valid for both windows and linux
      Log("Skipping Floppy test for WS")
      return

   # Add floppy
   Log("Add floppy")
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualFloppy)
   if len(devices) != 0:
      Log("Found floppy already present: %s" % str(devices))
      raise Exception("Floppy already present before add")

   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddFloppy(cspec, type="image", backingName=image)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualFloppy)
   if len(devices) != 1:
      raise Exception("Failed to find added floppy")
   PowerCycle(vm1)
   floppyDev = devices[0]

   # Reconfig floppy to point to a file backing. (Same type, different image)
   if about.productLineId != 'ws':
      image = "[] /vmimages/floppies/pvscsi-WindowsVista.flp"
   else:
      image = "[] /dev/null"

   Log("Reconfig floppy")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualFloppy.ImageBackingInfo()
   backing.SetFileName(image)
   floppyDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec, floppyDev, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualFloppy,
                                  {"backing.fileName": image})
   if len(devices) != 1:
      raise Exception("Failed to find edited floppy")
   PowerCycle(vm1)

   Log("Try to reconfigure floppy with invalid backing")
   cspec = Vim.Vm.ConfigSpec()
   #any backing of a device different than floppy could have been used
   backing = Vim.Vm.Device.VirtualSerialPort.PipeBackingInfo()
   floppyDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec,
                            floppyDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure floppy with invalid backing")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring floppy with invalid backing")

   #Reconfigure the floppy with file backing, with invalid file name.
   #This corresponds to vmops/reconfigvm/Neg038.java test from vcqe.
   Log("Try to reconfigure floppy with invalid file backing")
   cspec = Vim.Vm.ConfigSpec()
   backing = Vim.Vm.Device.VirtualFloppy.ImageBackingInfo()
   backing.SetFileName("xxx")
   floppyDev.SetBacking(backing)
   vmconfig.AddDeviceToSpec(cspec,
                            floppyDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDatastore as e:
      #on VC InvalidDataStore path is expected
      Log("Hit an expected exception when trying to configure floppy with invalid file")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring floppy with invalid file")

   #Reconfigure the floppy with NULL backing.
   #This corresponds to vmops/reconfigvm/Neg048.java test from vcqe.
   Log("Try to reconfigure floppy with NULL backing")
   cspec = Vim.Vm.ConfigSpec()
   floppyDev.SetBacking(None)
   vmconfig.AddDeviceToSpec(cspec,
                            floppyDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDeviceBacking as e:
      Log("Hit an expected exception when trying to configure floppy with NULL backing")
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure floppy with NULL backing")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when reconfiguring floppy with NULL backing")

   # Remove floppy
   Log("Remove floppy")
   vm.RemoveDevice(vm1, devices[0])
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualFloppy)
   if len(devices) != 0:
      raise Exception("Found floppy even after delete")
   PowerCycle(vm1)

def MessWithNics(vm1):
   # Add nic
   Log("Add nic")
   cspec = Vim.Vm.ConfigSpec()
   try:
      cspec = vmconfig.AddNic(cspec, cfgInfo = vm1.GetConfig())
   except Exception as e:
      if str(e) == "No device available on the host to add":
         Log("No network backings available, skipping Nic tests")
         return
      else:
         raise
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualEthernetCard)
   if len(devices) != 1:
      raise Exception("Failed to find added nic")
   PowerCycle(vm1)
   nicDev = devices[0]

   # Change address type to a specified mac
   Log("Edit nic to set wake on lan to something")
   cspec = Vim.Vm.ConfigSpec()
   # XXX: Unit test sets mac but I dont know what a good static mac is so temporarily do wake onlan
   nicDev.SetWakeOnLanEnabled(False)
#   nicDev.SetAddressType("Manual")
#   nicDev.SetMacAddress("00:0c:29:f6:06:d4")
   vmconfig.AddDeviceToSpec(cspec, nicDev, Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualEthernetCard,
                                  {"wakeOnLanEnabled": False})
#                                  {"macAddress": "00:0c:29:f6:06:d4"})
   if len(devices) != 1:
      raise Exception("Failed to find edited nic")
   PowerCycle(vm1)

   # Attempt to reconfigure a VM with a network card with invalid address type
   # This test reproduces vmops/reconfigvm/Neg024.java
   cspec = Vim.Vm.ConfigSpec()
   nicDev.SetAddressType("")
   vmconfig.AddDeviceToSpec(cspec,
                            nicDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure a network card with empty address type")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when trying to configure a network card with empty address type")

   # pretty much the same test but with an invalid string instead of empty one.
   # Neg024 doesn't cover this case.
   cspec = Vim.Vm.ConfigSpec()
   nicDev.SetAddressType("completely-invalid-address-type")
   vmconfig.AddDeviceToSpec(cspec,
                            nicDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Vim.Fault.InvalidDeviceSpec as e:
      Log("Hit an expected exception when trying to configure a network card with invalid address type")
   else:
      #no exception was raised, that is bad
      raise Exception("No exception was thrown when trying to configure a network card with invalid address type")

   # Test adding a fixed, vpx assigned MAC
   # This replicates part of VC/src/tests/java/vmops/createvm/Pos022.java
   cspec = Vim.Vm.ConfigSpec()
   nicDev.SetAddressType("assigned")
   nicDev.SetMacAddress("00:50:56:80:23:12")
   vmconfig.AddDeviceToSpec(cspec,
                            nicDev,
                            Vim.Vm.Device.VirtualDeviceSpec.Operation.edit)
   task = vm1.Reconfigure(cspec)
   try:
      WaitForTask(task)
   except Exception:
      raise Exception("Hit exception when trying to configure NIC with VPX assigned address")
   else:
      devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualEthernetCard,
                                     {"macAddress": "00:50:56:80:23:12",
                                      "addressType": "assigned"})
      if len(devices) != 1:
         raise Exception("Unable to find NIC configured with VPX assigned address")

   # Remove nic
   Log("Remove nic")
   vm.RemoveDevice(vm1, nicDev)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualEthernetCard)
   if len(devices) != 0:
      raise Exception("Found nic even after delete")
   PowerCycle(vm1)

def MessWithVideoCard(vm1, newName, datastore=None):
   Log("Edit video ram size")
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualVideoCard)
   if len(devices) < 1:
      raise Exception("Failed to find video card")
   video = devices[0]
   newVRamSize = 100 * 1024
   oldVRamSize = video.GetVideoRamSizeInKB()

   # Set invalid video ram sizes
   UpdateVideoRamSize(vm1, video, 10 , False) # too small
   UpdateVideoRamSize(vm1, video, 400 * 1024 , False) # too big

   if video.GetUseAutoDetect() == True:
      Log("Video card uses Auto-Detect mode. Skipping VRAM size tests.")
      return

   # Change the video ram size
   UpdateVideoRamSize(vm1, video, newVRamSize, True)
   # Restore old video ram size
   UpdateVideoRamSize(vm1, video, oldVRamSize, True)

   # Create a new VM with specified vRam Size
   Log("Creating new VM with specified vRam size")
   vm2 = vm.CreateQuickDummy(newName + "-video", 1, videoRamSize=newVRamSize, datastoreName=datastore)
   if vm2 == None:
      raise Exception("Failed to create VM with specified video ram size")
   devices = vmconfig.CheckDevice(vm2.GetConfig(), Vim.Vm.Device.VirtualVideoCard,
				  {"videoRamSizeInKB": newVRamSize})
   if len(devices) != 1:
      vm.Destroy(vm2)
      raise Exception("Newly created VM has incorrect video ram size")
   vm.Destroy(vm2)

def MessWithSnapshots(newName, datastore=None):
   vm2 = vm.CreateQuickDummy(newName, 1, datastoreName=datastore, memory=4)
   Log("Power On")
   vm.PowerOn(vm2)
   Log("Take a snapshot with memory")
   vm.CreateSnapshot(vm2, "Test Snapshot", None, True, False)
   Log("Power off")
   vm.PowerOff(vm2)

   Log("Revert to online snapshot while VM powered off")
   vm.RevertToCurrentSnapshot(vm2)

   Log("Revert to online snapshot while VM powered on")
   vm.RevertToCurrentSnapshot(vm2)

   bldType = os.environ.get('VMBLD', '') or os.environ.get('BLDTYPE', '')
   if bldType and bldType == 'obj':
      Log("Take a powered-on snapshot that will fail to power on upon revert")
      SetExtraConfig(vm2, "poweron.forceFail", "NIOC")
      vm.CreateSnapshot(vm2, "Test Snapshot Bad", None, True, False)

      Log("Revert to bad online snapshot while VM powered on")
      try:
         vm.RevertToCurrentSnapshot(vm2)
      except Exception as e:
         Log("Hit an exception when reverting to a bad snapshot as expected " +
             str(e))
      else:
         raise("Error: Reverting to a bad online snapshot was successful")

      SetExtraConfig(vm2, "poweron.forceFail", "")
      PowerCycle(vm2)
   else:
      vm.PowerOff(vm2)

   Log("Revert to online snapshot while offline and suppress power on")
   vm.RevertToCurrentSnapshot(vm2, True)

   vm2.Reload()
   if vm2.GetSnapshot() is None:
      raise Exception("Snapshot disappeared after reloading VM")
   snapInfo = vm2.GetSnapshot()
   currentSnap = snapInfo.GetCurrentSnapshot()
   vm.RemoveSnapshot(currentSnap)

   Log("Destroy vm")
   vm.Destroy(vm2)

def MessWithRecordReplay(newName, datastore=None):
   vm2 = vm.CreateQuickDummy(newName, guest="winXPProGuest", datastoreName=datastore)
   Log("Enable Record/Replay on the VM")
   cspec = Vim.Vm.ConfigSpec()
   flags = Vim.Vm.FlagInfo()
   flags.SetRecordReplayEnabled(True)
   cspec.SetFlags(flags)
   task = vm2.Reconfigure(cspec)
   WaitForTask(task)
   Log("Power On")
   vm.PowerOn(vm2)
   Log("Start recording")
   vm.StartRecording(vm2, "Recording1", "Test recording")
   time.sleep(10)
   Log("Stop recording")
   vm.StopRecording(vm2)
   snapInfo = vm2.GetSnapshot()
   rootSnapshotList = snapInfo.GetRootSnapshotList()
   rootSnapshot = rootSnapshotList[0]
   if rootSnapshot.GetReplaySupported() == False:
      raise Exception("Recorded Snapshot does not support replay!")
   Log("Start replaying")
   vm.StartReplaying(vm2, rootSnapshot.GetSnapshot())
   time.sleep(2)
   Log("Stop replaying")
   vm.StopReplaying(vm2)
   Log("Power Off")
   vm.PowerOff(vm2)
   Log("Destroy vm")
   vm.Destroy(vm2)

def MessWithRegister(vm1):
   # Loop run register tests because we want to catch vm leaks
   for i in range(2):
      vmname = vm1.GetConfig().GetName()
      cfg = vm1.GetConfig().GetFiles().GetVmPathName()
      Log("Unregistering: " + cfg)
      vm1.Unregister()
      success = 0
      try:
         PowerCycle(vm1)
      except Vmodl.Fault.ManagedObjectNotFound as e:
         success = 1
      if success == 0:
         raise Exception("Powercycle on unregistered vm worked")
      Log("Registering: " + cfg)
      folder.Register(cfg)
      vm1 = folder.Find(vmname)
      PowerCycle(vm1)

def MessWithFilePath(vm1):
   Log("Update file path VM")
   vmfiles = vm1.GetConfig().GetFiles()
   cspec = Vim.Vm.ConfigSpec(files = vmfiles)
   vm.Reconfigure(vm1, cspec)
   vmfiles2 = vm1.GetConfig().GetFiles()
   if vmfiles.snapshotDirectory != vmfiles2.snapshotDirectory or \
      vmfiles.suspendDirectory != vmfiles2.suspendDirectory or \
      vmfiles.logDirectory != vmfiles2.logDirectory:
      raise Exception("Failed to set vm files correctly")
   vm.PowerOn(vm1)
   vm.Reconfigure(vm1, cspec)
   vm.PowerOff(vm1)

def MessWithReload(vm1):
   Log("Reload VM")
   vmname = vm1.GetConfig().GetName()
   vm1.Reload()
   vm2 = folder.Find(vmname)
   if vm2 == None:
      raise Exception("Reload invalidates the VM")

def MessWithFirmware(vm1):
   firmwares = [Vim.Vm.GuestOsDescriptor.FirmwareType.bios,
                Vim.Vm.GuestOsDescriptor.FirmwareType.efi,
                Vim.Vm.GuestOsDescriptor.FirmwareType.csm]
   cspec = Vim.Vm.ConfigSpec()
   for firmware in firmwares:
      cspec.firmware = firmware
      WaitForTask(vm1.Reconfigure(cspec))
      config = vm1.config
      if config.firmware != firmware:
         raise Exception("Firmware not updated correctly %s != %s" %
                         (config.firmware, firmware))
      vm1.Reload()
      config = vm1.config
      if config.firmware != firmware:
         raise Exception("Firmware not read correctly after reload %s != %s" %
                         (config.firmware, firmware))

      if firmware == Vim.Vm.GuestOsDescriptor.FirmwareType.efi:
         if (config.bootOptions.networkBootProtocol !=
             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv4):
            raise Exception("Network boot protocol default not set %s != %s" %
                            (config.bootOptions.networkBootProtocol,
                             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv4))

         cspec.bootOptions = Vim.vm.BootOptions()
         cspec.bootOptions.networkBootProtocol = Vim.Vm.BootOptions.NetworkBootProtocolType.ipv6
         WaitForTask(vm1.Reconfigure(cspec))

         config = vm1.config
         if (config.bootOptions.networkBootProtocol !=
             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv6):
            raise Exception("Network boot protocol not updated correctly %s "
                            "!= %s" %
                            (config.bootOptions.networkBootProtocol,
                             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv6))

         vm1.Reload()
         config = vm1.config
         if (config.bootOptions.networkBootProtocol !=
             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv6):
            raise Exception("Network boot protocol not read correctly after "
                            "reload %s != %s" %
                            (config.bootOptions.networkBootProtocol,
                             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv6))

         ## Set back to default so someone can run this test back to back.
         cspec.bootOptions.networkBootProtocol = Vim.Vm.BootOptions.NetworkBootProtocolType.ipv4
         WaitForTask(vm1.Reconfigure(cspec))

         config = vm1.config
         if (config.bootOptions.networkBootProtocol !=
             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv4):
            raise Exception("Network boot protocol not updated correctly %s "
                            "!= %s" %
                            (config.bootOptions.networkBootProtocol,
                             Vim.Vm.BootOptions.NetworkBootProtocolType.ipv4))


def MessWithPassthru(vm1):
   def CheckDeviceBacking(config, deviceType, propMap = None):
      devices = []
      for device in config.GetHardware().GetDevice():
         if isinstance(device, deviceType):
            prop = vmconfig.CheckPropMap(device.GetBacking(), propMap)
            if prop == None:
               devices.append(device)
            else:
               print("Did not match property: %s" % prop)
               print("Device %s propMap %s" % (device, backingDict))
      return devices
   # Gibberish values since power-on is expected to fail, but exercise all types
   # Format is [(TYPE, dict), ...]
   backings = [(Vim.Vm.Device.VirtualPCIPassthrough.DeviceBackingInfo,
                { 'id':"0000:01:02.3", 'deviceId':"1234", 'systemId':"system-id",
                  # 'vendorId':1234, (disabled - type mismatch, vmomi.short != int)
                }),
               (Vim.Vm.Device.VirtualPCIPassthrough.VmiopBackingInfo,
                { 'vgpu':"gpu_k100", }),
              ]
   for backingType, backingDict in backings:
      Log("Add passthru backing " + backingType.__name__)
      cspec = Vim.Vm.ConfigSpec()
      passthru = Vim.Vm.Device.VirtualPCIPassthrough()
      passthru.SetKey(vmconfig.GetFreeKey(cspec))
      passthru.backing = backingType(**backingDict)

      vmconfig.AddDeviceToSpec(cspec, passthru, Vim.Vm.Device.VirtualDeviceSpec.Operation.add)
      WaitForTask(vm1.Reconfigure(cspec))
      devices = CheckDeviceBacking(vm1.GetConfig(), Vim.Vm.Device.VirtualPCIPassthrough,
                                   backingDict)
      print(devices)
      if len(devices) != 1:
         raise Exception("Failed to find added passthru")

      vm1.Reload()
      devices = CheckDeviceBacking(vm1.GetConfig(), Vim.Vm.Device.VirtualPCIPassthrough,
                                   backingDict)
      if len(devices) != 1:
         raise Exception("Failed to find edited passthru after reload")

      Log("Remove passthru")
      vm.RemoveDevice(vm1, devices[0])
      devices = CheckDeviceBacking(vm1.GetConfig(), Vim.Vm.Device.VirtualPCIPassthrough)
      if len(devices) != 0:
         raise Exception("Found passthru even after delete")


def MessWithDefaultPowerOps(vm1):
   current = vm1.config.defaultPowerOps.powerOffType
   new = 'hard' if current == 'soft' else 'soft'
   powerOpInfo = Vim.Vm.DefaultPowerOpInfo(powerOffType=new)
   task = vm1.Reconfigure(Vim.Vm.ConfigSpec(powerOpInfo=powerOpInfo))
   WaitForTask(task)
   updated = vm1.config.defaultPowerOps.powerOffType
   if updated != new:
      raise Exception('DefaultPowerOffType not updated correctly: "%s" != "%s"' %
                      (updated, new))

   current = vm1.config.defaultPowerOps.standbyAction
   new = 'powerOnSuspend' if current == 'checkpoint' else 'checkpoint'
   powerOpInfo = Vim.Vm.DefaultPowerOpInfo(standbyAction=new)
   task = vm1.Reconfigure(Vim.Vm.ConfigSpec(powerOpInfo=powerOpInfo))
   WaitForTask(task)
   updated = vm1.config.defaultPowerOps.standbyAction
   if updated != new:
      raise Exception('StandbyAction not updated correctly: "%s" != "%s"' %
                      (updated, new))

def MessWithBios(vm1, datastore):
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiCtlr(cspec)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore)

   global about
   if about.productLineId != 'ws':
      image = "[] /vmimages/floppies/vmscsi.flp"
      cspec = vmconfig.AddFloppy(cspec, type="image", backingName=image)

   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   floppyDevices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualFloppy)
   diskDevices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)

   spec = Vim.vm.ConfigSpec()
   spec.bootOptions = Vim.vm.BootOptions()
   spec.bootOptions.bootOrder = [
      Vim.vm.BootOptions.BootableDiskDevice(deviceKey=diskDevices[1].key),
      Vim.vm.BootOptions.BootableDiskDevice(deviceKey=diskDevices[0].key),
                                ]
   if len(floppyDevices):
      spec.bootOptions.bootOrder.append(Vim.vm.BootOptions.BootableFloppyDevice())
   spec.bootOptions.enterBIOSSetup = True
   spec.bootOptions.bootRetryEnabled = True
   spec.bootOptions.bootRetryDelay = long2to3(1000)
   spec.bootOptions.bootDelay = long2to3(1)
   spec.firmware = 'efi'

   # ESX only
   if about.productLineId != 'ws':
      spec.bootOptions.efiSecureBootEnabled = True

   WaitForTask(vm1.Reconfigure(spec))
   vm.PowerOn(vm1)

   options = vm1.config.bootOptions

   if not IsBootOrderEqual(spec.bootOptions.bootOrder,
                           vm1.config.bootOptions.bootOrder):
      raise Exception("Boot order does not match, spec: %s, options: %s"
                      % (spec.bootOptions, options))
   time.sleep(5)
   vm.PowerOff(vm1)

   # Attempt to reconfigure a few of the BIOS parameters online
   spec = Vim.vm.ConfigSpec()
   spec.bootOptions = Vim.vm.BootOptions()
   spec.bootOptions.bootRetryEnabled = False
   spec.bootOptions.bootRetryDelay = long2to3(1001)
   spec.bootOptions.bootDelay = long2to3(2)

   vm.PowerOn(vm1)
   WaitForTask(vm1.Reconfigure(spec))

   if (vm1.config.bootOptions.bootRetryEnabled !=
       spec.bootOptions.bootRetryEnabled):
      raise Exception("Unexpected reconfigure bootRetryEnabled value")

   if vm1.config.bootOptions.bootRetryDelay != spec.bootOptions.bootRetryDelay:
      raise Exception("Unexpected reconfigure bootRetryDelay value")

   if vm1.config.bootOptions.bootDelay != spec.bootOptions.bootDelay:
      raise Exception("Unexpected reconfigure bootDelay value")

   # Check that efiSecureBootEnabled is not hot settable. ESX only
   if about.productLineId != 'ws':
      spec = Vim.vm.ConfigSpec()
      spec.bootOptions = Vim.vm.BootOptions()
      spec.bootOptions.efiSecureBootEnabled = False
      task = vm1.Reconfigure(spec)
      WaitForTask(task, raiseOnError=False)
      if not isinstance(task.info.error, Vim.Fault.InvalidPowerState):
         raise Exception("Hot set of efiSecureBootEnabled succeeded.")

   vm.PowerOff(vm1)

   """
   options = vm1.config.bootOptions
   if spec.bootOptions.enterBIOSSetup == options.enterBIOSSetup:
      raise Exception("Enter BIOS Setup was unchanged after boot, spec: %s, options: %s"
                      % (spec.bootOptions, options))
   """

def IsBootOrderEqual(lhs, rhs):
   if len(lhs) != len(rhs):
      Log("Length mismatch %d != %d" % (len(lhs), len(rhs)))
      return False
   for i in range(len(lhs)):
      if type(lhs[i]) != type(rhs[i]):
         Log("Type mismatch %s != %s" % (type(lhs[i]), type(rhs[i])))
         return False
      if isinstance(lhs[i], Vim.vm.BootOptions.BootableDiskDevice):
         if lhs[i].deviceKey != rhs[i].deviceKey:
            Log("Device Key mismatch %d != %d" % (lhs[i].deviceKey, rhs[i].deviceKey))
            return False
   return True

def MessWithSwapPlacement(vm1, si, datastore):
   if about.productLineId == 'ws':
      Log("Skipping swap placement tests for WS")
      return

   current = vm1.config.swapPlacement
   new = 'hostLocal' if current == 'inherit' else 'inherit'
   task = vm1.Reconfigure(Vim.Vm.ConfigSpec(swapPlacement=new))
   WaitForTask(task)
   updated = vm1.config.swapPlacement
   if updated != new:
      raise Exception('SwapPlacement not updated correctly: "%s" != "%s"' %
                      (updated, new))
   if new == 'inherit':
      task = vm1.Reconfigure(Vim.Vm.ConfigSpec(swapPlacement='hostLocal'))
      WaitForTask(task)
   # enable host local datastore
   configMgr = host.GetHostConfigManager(si)
   dsSystem = configMgr.GetDatastoreSystem()
   hostSystem = host.GetHostSystem(si)
   for ds in hostSystem.datastore:
      if ds.summary.type == 'VMFS':
         dsSystem.UpdateLocalSwapDatastore(ds)
   # power on the VM and hot-add a controller
   vm.PowerOn(vm1)
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddSataCtlr(cspec)
   cspec = vmconfig.AddSataDisk(cspec, datastorename=datastore)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   ctlrs = vmconfig.CheckDevice(vm1.GetConfig(),
                                Vim.Vm.Device.VirtualSATAController)
   if len(ctlrs) != 1:
      raise Exception("Failed to find added SATA controller!")
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk,
                                  {"controllerKey": ctlrs[0].key})
   if len(devices) < 1:
      raise Exception("Failed to find added SATA disk!")
   disk = devices[0]
   # clean up
   vm.PowerOff(vm1)
   cspec = Vim.Vm.ConfigSpec()
   fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
   vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
   vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   dsSystem.UpdateLocalSwapDatastore()

def MessWithScheduledHardwareUpgrade(newName, datastore):
   # Add power state change after bug 947025
   Log("Creating new VM for hardware upgrade schedule")
   vm2 = vm.CreateQuickDummy(newName + "-upgrade", 1, datastoreName=datastore, vmxVersion = "vmx-07")
   upgradeSpec = Vim.Vm.ScheduledHardwareUpgradeInfo()
   policy = Vim.Vm.ScheduledHardwareUpgradeInfo.HardwareUpgradePolicy.always
   upgradeSpec.upgradePolicy = policy
   cspec = Vim.Vm.ConfigSpec()
   cspec.scheduledHardwareUpgradeInfo = upgradeSpec
   WaitForTask(vm2.Reconfigure(cspec))

   upgradeInfo = vm2.config.scheduledHardwareUpgradeInfo
   if upgradeInfo.upgradePolicy != policy:
         raise Exception("Upgrade policy not updated correctly")
   Log("Scheduled upgrade version key is: " + upgradeInfo.versionKey)

   upgradeSpec.versionKey = "vmx-05"
   try:
      WaitForTask(vm2.Reconfigure(cspec))
   except Exception as e:
      Log("Hit an exception on upgrade to an unsupported as expected" + str(e))
   else:
      raise("Error: upgrade to unsupported version should not be allowed")

   upgradeSpec.versionKey = None
   policy = Vim.Vm.ScheduledHardwareUpgradeInfo.HardwareUpgradePolicy.onSoftPowerOff
   upgradeSpec.upgradePolicy = policy
   WaitForTask(vm2.Reconfigure(cspec))
   upgradeInfo = vm2.config.scheduledHardwareUpgradeInfo
   if upgradeInfo.upgradePolicy != policy:
         raise Exception("Upgrade policy not updated correctly")

   upgradeSpec.versionKey = "vmx-04"
   try:
      WaitForTask(vm2.Reconfigure(cspec))
   except Exception as e:
      Log("Hit an exception on upgrade to existing  as expected" + str(e))
   else:
      Log("Error: upgrade to same version should not be allowed")

   Log("Destroy the scheduled hardware version VM")
   vm.Destroy(vm2)

def SetExtraConfig(vm1, key, value):
   cspec = Vim.Vm.ConfigSpec(extraConfig=[Vim.Option.OptionValue(key=key, value=value)])
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

def GetExtraConfig(vm1, key):
   try:
      return [o.value for o in vm1.config.extraConfig if o.key == key][0]
   except IndexError:
      raise KeyError(key)

def CheckExtraConfig(vm1, key, value):
   newValue = GetExtraConfig(vm1, key)
   if value != newValue:
      raise Exception('ExtraConfig key "%s", expected value "%s", returned value "%s"',
                      key, value, newValue)

def MessWithExtraConfig(vm1):
   for i in range(2):
      # Add a valid extraConfig option, verify that it comes back correctly
      (key, value) = ('case.color', 'white')
      SetExtraConfig(vm1, key, value)
      CheckExtraConfig(vm1, key, value)
      # Reset
      SetExtraConfig(vm1, key, "")

      # Add a valid option with integer, verify that it comes back correctly
      (key, value) = ('case.length', 11)
      SetExtraConfig(vm1, key, value)
      CheckExtraConfig(vm1, key, str(value))
      # Reset
      SetExtraConfig(vm1, key, "")

      # Add a filtered extraConfig option, verify that it's a no-op
      (key, value) = ('bios.bootdelay', '17')
      SetExtraConfig(vm1, key, value)
      try:
         GetExtraConfig(vm1, key)
         raise Exception('ExtraConfig filter on key "%s" failed' % key)
      except KeyError:
         pass

      # Add an invalid extraConfig option, verify that the reconfigure fails
      (key, value) = ('==>', 'x')
      try:
         SetExtraConfig(vm1, key, value)
         raise Exception('ExtraConfig with invalid key "%s" succeeded' % key)
      except Vmodl.Fault.InvalidArgument:
         pass

      # Add migrate.hostLog with None value, verify InvalidArgument is thrown
      (key, value) = ('migrate.hostLog', None)
      try:
         SetExtraConfig(vm1, key, value)
      except Vmodl.Fault.InvalidArgument:
         pass # InvalidArgument is expected
      else:
         raise Exception('Reconfigure succeeded with %s=%s' % (key, value))

      # Add migrate.hostLog with int value, verify InvalidArgument is thrown
      (key, value) = ('migrate.hostLog', 1)
      try:
         SetExtraConfig(vm1, key, value)
      except Vmodl.Fault.InvalidArgument:
         pass # InvalidArgument is expected
      else:
         raise Exception('Reconfigure succeeded with %s=%s' % (key, value))

      # Test changing sched.swap.derivedName does not make the VM invalid
      if vm1.runtime.powerState == Vim.VirtualMachine.PowerState.poweredOn:
         (key, value) = ('sched.swap.derivedName', '/bin/true')
         SetExtraConfig(vm1, key, value)
         # sleep a little to allow for the config check to be triggered
         time.sleep(5)

      if i == 0:
         vm.PowerOn(vm1)
      else:
         vm.PowerOff(vm1)

   # Add an OVF environment, verify that the value is cleared on read
   # XXX disabling this test for now since it's not clear it's doing the
   # Right Thing (see PR954605)
   #(key, value) = ('guestInfo.ovfEnv', 'hello')
   #SetExtraConfig(vm1, key, value)
   #CheckExtraConfig(vm1, key, '')

def MessWithFork(vm1):
   global about
   if about.productLineId == 'ws':
      Log("Skipping vmfork tests for WS")
      return

   # Add virtual disk to VM
   # Set/unset childGroupId, parentGroupId, with powerOn
   uuid = "abcd0123-4567-ef89-0123-abcdefabcdef"
   forkConfigInfo = Vim.vm.ForkConfigInfo(childType="nonpersistent",
                                          childForkGroupId=uuid)
   spec = Vim.vm.InternalConfigSpec(internalForkConfigInfo=forkConfigInfo)

   WaitForTask(vm1.Reconfigure(spec))
   assert vm1.config.forkConfigInfo.childForkGroupId == uuid

   forkConfigInfo.childForkGroupId = None
   forkConfigInfo.childType = "none"
   WaitForTask(vm1.Reconfigure(spec))
   assert vm1.config.forkConfigInfo.childForkGroupId == None
   assert vm1.config.forkConfigInfo.childType == None

   forkConfigInfo.childType = None
   forkConfigInfo.parentForkGroupId = uuid
   forkConfigInfo.parentEnabled = True

   WaitForTask(vm1.Reconfigure(spec))
   assert vm1.config.forkConfigInfo.parentForkGroupId == uuid
   assert vm1.config.forkConfigInfo.childType == None
   vm.PowerOn(vm1)
   assert vm1.config.forkConfigInfo.parentForkGroupId == uuid
   assert vm1.config.forkConfigInfo.parentEnabled == True
   forkConfigInfo.parentEnabled = False
   WaitForTask(vm1.Reconfigure(spec))
   assert vm1.config.forkConfigInfo.parentForkGroupId == None
   assert vm1.config.forkConfigInfo.parentEnabled == False
   assert vm1.config.forkConfigInfo.childType == None

   vm.PowerOff(vm1)
   assert vm1.config.forkConfigInfo.parentForkGroupId == None
   assert vm1.config.forkConfigInfo.childType == None

def GetDatastore(si, dsName):
   hs = host.GetHostSystem(si)
   datastores = hs.GetDatastore()
   for ds in datastores:
      if ds.name == dsName:
         return ds
   raise Exception("Error looking up datastore: %s" % dsName)

def RetryFileOp(*args, **kwargs):
   retry = 2
   while True:
      time.sleep(retry)
      try:
         vimutil.InvokeAndTrack(*args, **kwargs)
         break
      except (Vim.fault.FileFault, Vim.fault.FileLocked):
         if retry == 5:
            raise
         retry += 1

def MessWithBadParentDisk(vm1, datastore, newName, si):
   # Test that storage error for a parent disk that results in an incomplete
   # file layout would not cause the VmxSandbox disk path sanity check to
   # ASSERT.
   Log("Add a new disk to the VM")
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiCtlr(cspec)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore)
   vimutil.InvokeAndTrack(vm1.Reconfigure, cspec)
   devices = vmconfig.CheckDevice(vm1.GetConfig(), Vim.Vm.Device.VirtualDisk)
   if len(devices) < 2:
      raise Exception("Failed to find added disk!")
   disk = devices[1]
   ctlrKey = disk.GetControllerKey()
   ctlrs = vmconfig.CheckDevice(vm1.GetConfig(),
                                Vim.Vm.Device.VirtualSCSIController,
				{"key": ctlrKey})
   if len(ctlrs) != 1:
      raise Exception("Failed to find added SCSI controller!")
   Log("Take a snapshot")
   vm.CreateSnapshot(vm1, "s1", "s1", False, False)
   s1 = vm1.GetSnapshot().GetCurrentSnapshot()
   parentDisk = vmconfig.CheckDevice(s1.GetConfig(),
                                     Vim.Vm.Device.VirtualDisk)[1]
   parentDiskPath = parentDisk.GetBacking().GetFileName()
   vmxPath = vm1.GetSummary().GetConfig().GetVmPathName()
   if datastore is None:
      m = re.search(r'\[(.+)\]', vmxPath)
      datastore = m.group(1)
   Log("Create test child VM")
   vmName = newName + '_c'
   # Destroy previously used VM
   vm2 = folder.Find(vmName)
   if vm2 != None:
      vm.Destroy(vm2)
   cspec = vmconfig.CreateDefaultSpec(name=vmName, datastoreName=datastore)
   cspec = vmconfig.AddScsiCtlr(cspec)
   cspec = vmconfig.AddScsiDisk(cspec, datastorename=datastore, capacity=1024)
   diskBacking = cspec.GetDeviceChange()[1].GetDevice().GetBacking()
   parentBacking = Vim.Vm.Device.VirtualDisk.FlatVer2BackingInfo()
   parentBacking.SetFileName(parentDiskPath)
   diskBacking.SetParent(parentBacking)
   ds = GetDatastore(si, datastore)
   if isinstance(ds.info, Vim.Host.VmfsDatastoreInfo) and \
      ds.info.vmfs.majorVersion >= 6:
      diskBacking.deltaDiskFormat = 'seSparseFormat'
   else:
      diskBacking.SetDeltaDiskFormat('redoLogFormat')
   hsi = Hostd.ServiceInstance("ha-internalsvc-service-instance", si._stub)
   # Inject error so that child VM would not populate external disk path cache
   debugMgr = hsi.GetDebugManager()
   debugMgr.InjectDiskLayoutError(True, True)
   resPool = invt.GetResourcePool()
   vmFolder = invt.GetVmFolder()
   vimutil.InvokeAndTrack(vmFolder.CreateVm, cspec, resPool)
   vm2 = folder.Find(vmName)
   Log("Test that child VM can be powered on witout up-to-date layout")
   vm.PowerOn(vm2)
   debugMgr.ForceDiskPathSanityCheck()
   vm.PowerOff(vm2)
   debugMgr.InjectDiskLayoutError(True, False)
   # Test realpath error would not cause VM to become invalid
   Log("Power on child VM to test realpath error (datastore = %s)" % datastore)
   # Make sure there is a symbolic path
   SetExtraConfig(vm2, "vmx.fileTrack.logFile",
                  "/vmfs/volumes/%s/fileTrack.log" % datastore)
   vm.PowerOn(vm2)
   debugMgr.TestRealpathError(True)
   vm.PowerOff(vm2)
   debugMgr.TestRealpathError(False)
   Log("Corrupt the parent disk descriptor")
   fileManager = GetSi().RetrieveContent().GetFileManager()
   vimutil.InvokeAndTrack(fileManager.CopyFile, sourceName=parentDiskPath,
                          destinationName=parentDiskPath + '.bk', force=True)
   nvramPath = vmxPath.replace('.vmx', '.nvram')
   RetryFileOp(fileManager.CopyFile, sourceName=nvramPath,
               destinationName=parentDiskPath, force=True)
   # Reload so file layout would be missing parent disk
   vm2.Reload()
   debugMgr.ForceDiskPathSanityCheck()
   try:
      vm.PowerOn(vm2)
      raise Exception("Power-on succeeded unexpectedly for child VM")
   except Vim.Fault.GenericVmConfigFault as e:
      pass
   # Destroy child vm and remove added disk + controller from parent
   vm.Destroy(vm2)
   RetryFileOp(fileManager.MoveFile, sourceName=parentDiskPath + '.bk',
               destinationName=parentDiskPath, force=True)
   vm.RemoveSnapshot(s1)
   cspec = Vim.Vm.ConfigSpec()
   fileOp = Vim.Vm.Device.VirtualDeviceSpec.FileOperation.destroy
   vmconfig.RemoveDeviceFromSpec(cspec, disk, fileOp)
   vmconfig.RemoveDeviceFromSpec(cspec, ctlrs[0])
   vimutil.InvokeAndTrack(vm1.Reconfigure, cspec)
   # Power cycle the parent to make sure it works
   PowerCycle(vm1)

def MessWithBadReconfig(vm1):
   # Test an online reconfigure that crashes the vmx would not crash hostd.
   SetExtraConfig(vm1, "vmx.test.exitAfterEndBatch", "TRUE")
   vm.PowerOn(vm1)
   cspec = Vim.Vm.ConfigSpec()
   vmconfig.AddScsiDisk(cspec, cfgInfo=vm1.config)
   try:
      vimutil.InvokeAndTrack(vm1.Reconfigure, cspec)
   except Vim.Fault.GenericVmConfigFault as e:
      pass
   # Wait for the VM to transition to powered off
   for i in range(10):
      if vm1.runtime.powerState == Vim.VirtualMachine.PowerState.poweredOff:
         break
      elif i == 9:
         raise Exception("VM has not transitioned to poweredOff")
      time.sleep(1)
   SetExtraConfig(vm1, "vmx.test.exitAfterEndBatch", "")
   PowerCycle(vm1)

def HostdMemSize(si):
   # Get hostd virtual memory size in KB
   # Note: This indirectly test the SystemDebugManager, and PyVmomiServer code
   #       used by SystemDebugManager
   hostSystem = host.GetHostSystem(si)
   internalCfgMgr = hostSystem.RetrieveInternalConfigManager()
   if not internalCfgMgr.systemDebugManager:
      return 0

   procInfos = internalCfgMgr.systemDebugManager.QueryProcessInfo()
   for procInfo in procInfos:
      if procInfo.processKey == 'hostd':
         return procInfo.virtualMemSize
   return 0

def BrowseDir(datastore, path, search='*'):
   browse = datastore.browser
   searchSpec = Vim.Host.DatastoreBrowser.SearchSpec()
   searchSpec.matchPattern = search
   task = browse.Search(path, searchSpec)
   WaitForTask(task)
   result = []
   for file in task.info.result.file:
      result.append(file)
   return result

def DirectoryExists(datastores, directory):
   for datastore in datastores:
      try:
         if directory.startswith('[%s]' % datastore.name) and len(BrowseDir(datastore, directory)):
            return True
      except Vim.Fault.FileNotFound as e:
         pass
   return False

def AreAllVmFilesDeleted(datastores, fileInfos):
   #Helper function to check that all files belonging to a VM are deleted. If all files
   #are gone it returns True, otherwise it returns False.
   for datastore in datastores:
      for fileInfo in fileInfos:
         try:
            if fileInfo.name.startswith('[%s]' % datastore.name) and len(BrowseDir(datastore, fileInfo.name)):
               return False
         except Vim.Fault.FileNotFound as e:
            pass
   return True


def MessWithDestroy(newName, datastore=None, hostName=None):
   #Create a VM with 2 disks and 3 snapshots then destroy it
   #and make sure all files are gone
   vm1 = vm.CreateQuickDummy(newName, guest="winXPProGuest", datastoreName=datastore)

   #add one disk, take a snapshot, power on, take a running snapshot.
   cspec = Vim.Vm.ConfigSpec()
   cspec = vmconfig.AddScsiCtlr(cspec)
   vmconfig.AddScsiDisk(cspec, datastorename=datastore, thin=True)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   vm.CreateSnapshot(vm1, "DummySnapshot1", "dummy desc", False, False)
   vm.PowerOn(vm1)
   vm.CreateSnapshot(vm1, "DummySnapshot2", "dummy desc", False, False)
   vm.PowerOff(vm1)

   #Change file locations, reconfigure
   #The directorise we will use need to exist, so we'll have to create them
   #IMPORTANT On ESX50 suspendDir is ignored and snapshotDir is used instead.
   #see FileInfo VMODL class for detailed info.
   files = Vim.Vm.FileInfo()
   pathBase = os.path.dirname(vm1.GetConfig().GetFiles().GetVmPathName())
   suspendDir = pathBase + "-suspends"
   snapshotDir = pathBase +"-snapshots"
   directories = [suspendDir, snapshotDir]
   fileManager = GetSi().RetrieveContent().GetFileManager()
   for directory in directories:
      try:
         fileManager.MakeDirectory(directory)
      except Vim.Fault.FileAlreadyExists as e:
         pass
   files.SetSuspendDirectory(suspendDir)
   files.SetSnapshotDirectory(snapshotDir)
   cspec.SetFiles(files)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)

   #add another disk, create a snapshot. It should go into new location.
   #Powercycle just in case
   vmconfig.AddScsiDisk(cspec, datastorename=datastore, thin=True)
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   vm.CreateSnapshot(vm1, "DummySnapshot3", "dummy desc", False, False)
   PowerCycle(vm1)

   #now we'll suspend the VM to create some more files
   vm.PowerOn(vm1)
   vm.Suspend(vm1)

   #time to get the list of all files belonging to VM and destroy it.
   vm1.RefreshStorageInfo()
   fileInfos = vm1.GetLayoutEx().GetFile()
   datastores = vm1.GetDatastore()
   vm.Destroy(vm1)
   if AreAllVmFilesDeleted(datastores, fileInfos) == False:
      raise Exception("Destroying VM didn't remove all files!")

   # we'll cleanup the dirs we have created. If something is left there
   # DirectoryExists will return true and will throw up and
   # we know we have a failed test
   for directory in directories:
      try:
         task = fileManager.Delete(datastorePath=directory,
                                   fileType=Vim.FileManager.FileType.File)
         WaitForTask(task)
      except Vim.Fault.FileNotFound as e:
         pass
      if DirectoryExists(datastores, directory):
         raise Exception("Directory %s not removed." % directory)

   #as a final test of destroy feature we will just create a simple VM and destroy it
   #right away. Tests above cover the more complicated cases - we shouldn't forget the
   #easy ones though.
   vm2 = vm.CreateQuickDummy(newName, guest="winXPProGuest", datastoreName=datastore)
   fileInfos = vm2.GetLayoutEx().GetFile()
   datastores = vm2.GetDatastore()
   vm.Destroy(vm2)
   if AreAllVmFilesDeleted(datastores, fileInfos) == False:
      raise Exception("Destroying VM didn't remove all files!")


def MessWithEventFilterSpec():
   eventManager = GetSi().RetrieveContent().GetEventManager()
   VerifyEventFilterByEventTypeId(eventManager)
   VerifyEventFilterByType(eventManager)

def MessWithCbrcDigesCache(vm1):
   if about.productLineId == 'ws':
      Log("Skipping cbrc tests for WS")
      return

   assert vm1.config.flags.cbrcCacheEnabled == False

   cspec = Vim.Vm.ConfigSpec()
   cspec.flags = Vim.vm.FlagInfo()
   cspec.flags.cbrcCacheEnabled = True
   WaitForTask(vm1.Reconfigure(cspec))
   assert vm1.config.flags.cbrcCacheEnabled == True

   vm.PowerOn(vm1)
   assert vm1.config.flags.cbrcCacheEnabled == True
   vm.PowerOff(vm1)

   cspec.flags.cbrcCacheEnabled = False
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   assert vm1.config.flags.cbrcCacheEnabled == False
   vm.PowerOn(vm1)
   assert vm1.config.flags.cbrcCacheEnabled == False

   cspec.flags.cbrcCacheEnabled = True
   task = vm1.Reconfigure(cspec)
   WaitForTask(task, raiseOnError=False)
   assert type(task.info.error) is Vim.fault.InvalidPowerState

   vm.PowerOff(vm1)

def VerifyEventFilterByEventTypeId(eventManager):
   typeIdMap = [("UserLoginSessionEvent", Vim.Event.UserLoginSessionEvent),
                ]

   for (typeId, cls) in typeIdMap:
      eventFilterSpec = Vim.Event.EventFilterSpec()
      eventFilterSpec.SetEventTypeId([typeId])

      collector = eventManager.CreateCollector(eventFilterSpec)
      eventList = collector.ReadNext(100)
      collector.Remove()

      if not eventList:
         raise Exception("Empty event list; filtered by eventTypeId: %r" %
                         typeId)

      for event in eventList:
         if type(event) != cls:
            raise Exception("Unexpected event type; got %s, expected: %s" %
                            (type(event), cls))

         if isinstance(event, Vim.Event.EventEx):
            if event.eventTypeId != typeId:
               raise Exception("Unexpected eventTypeId; got %r, expected: %r" %
                               (event.eventTypeId, typeId))


def VerifyEventFilterByType(eventManager):
   types = [Vim.Event.UserLoginSessionEvent,
              ]
   eventFilterSpec = Vim.Event.EventFilterSpec()
   eventFilterSpec.SetType(types)

   eventList = None
   numEvents = 100
   try:
      collector = eventManager.CreateCollector(eventFilterSpec)
      eventList = collector.ReadNext(numEvents)
      collector.Remove()
   except:
      pass
   if not eventList:
      raise Exception("Empty event list; filtered by type: %s" %
                      types)
   for event in eventList:
      for expectedType in types:
         if not isinstance(event, expectedType):
            raise Exception("Unexpected event type; %s, expected: %s" %
                            (event.__class__, types))

def MessWithGuestIntegrityInfo(vm1):
   if about.productLineId == 'ws':
      Log("Skipping guest integrity info tests for WS")
      return

   assert vm1.config.guestIntegrityInfo.enabled == False

   cspec = Vim.Vm.InternalConfigSpec()
   cspec.internalGuestIntegrityInfo = Vim.Vm.GuestIntegrityInfo()

   cspec.internalGuestIntegrityInfo.enabled = True
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   assert vm1.config.guestIntegrityInfo.enabled == True

   vm.PowerOn(vm1)
   assert vm1.config.guestIntegrityInfo.enabled == True

   vm.PowerOff(vm1)
   cspec.internalGuestIntegrityInfo.enabled = False
   task = vm1.Reconfigure(cspec)
   WaitForTask(task)
   assert vm1.config.guestIntegrityInfo.enabled == False

   vm.PowerOn(vm1)
   assert vm1.config.guestIntegrityInfo.enabled == False

   cspec.internalGuestIntegrityInfo.enabled = True
   task = vm1.Reconfigure(cspec)
   WaitForTask(task, raiseOnError=False)
   assert type(task.info.error) is Vim.fault.InvalidPowerState

   vm.PowerOff(vm1)

# XXX: Add power state, snapshot state, device state validations
def RunTests(argv):
   global about

   # Can't use built-in help option because -h is used for hostname
   parser = OptionParser(add_help_option=False)

   parser.add_option('-h', '--host', dest='host', help='Host name', default='localhost')
   parser.add_option('-u', '--user', dest='user', help='User name', default='root')
   parser.add_option('-p', '--pwd', dest='pwd', help='Password')
   parser.add_option('-o', '--port', dest='port', help='Port',
                     default=443, type='int')
   parser.add_option('-i', '--numiter', dest='iter', help='Iterations',
                     default=1, type='int')
   parser.add_option('-d', '--datastore', dest='datastore', help='Datastore')
   parser.add_option('-v', '--volume', dest='volume', help='VMFS volume')
   parser.add_option('-c', '--cleanup', dest='cleanup',
                     help='Try to clean up test VMs from previous runs',
                     default=False, action='store_true')
   parser.add_option('-?', '--help', '--usage', action='help', help='Usage information')

   (options, args) = parser.parse_args(argv)

   # Connect
   si = SmartConnect(host=options.host,
                     user=options.user,
                     pwd=options.pwd,
                     port=options.port)
   atexit.register(Disconnect, si)
   about = si.content.about
   status = "PASS"

   thread = MonitorProperties(si)
   thread.setDaemon(True)
   thread.start()

   # Determine the datastore name
   if options.datastore and options.volume:
      Log("Cannot specify both datastore and volume")
      sys.exit(-1)
   elif options.datastore:
      datastore = options.datastore
      Log("Using specified datastore: '%s'" % datastore)
   elif options.volume:
      # Convert absolute volume path to (dsName, relPath) tuple to get dsName
      Log("Trying to determine datastore name for volume: '%s'" % options.volume)
      try:
         datastore = path.FsPathToTuple(options.volume)[0]
         Log("Using datastore: '%s'" % datastore)
      except path.FilePathError as e:
         Log("Could not determine datastore name for volume: '%s'" % options.volume)
         sys.exit(-1)
   else:
      Log("No datastore or volume specified, test will choose one")
      datastore = None

   if not IsMockup():
      Log("Hostd memory : %d KB" % HostdMemSize(si))

   resultsArray = []

   for i in range(options.iter):
      bigClock = StopWatch()
      try:
         try:
            # Find the vm without tools
            newName = VM_NAME_PREFIX + '_' + str(i)
            vm2 = folder.Find(newName)
            if vm2 != None:
               if options.cleanup:
                  Log("Cleaning up old vm with name: " + newName)
                  try:
                     vm.PowerOff(vm2)
                  except: pass
                  vm.Destroy(vm2)
               else:
                  Log("Please cleanup unit test vms from previous runs")
                  sys.exit(-1)
            ph = Phase()

            # Create a simple vm
            vm2 = vm.CreateQuickDummy(newName, 1, datastoreName=datastore)
            ph.SetPhase("Create VM on " + newName)

            # Do reload tests
            MessWithReload(vm2)
            ph.SetPhase("Reload on " + newName)

            # Do file path tests
            MessWithFilePath(vm2)
            ph.SetPhase("File path reconfigure on " + newName)

            # UUID tests
            MessWithUuid(vm2)
            ph.SetPhase("UUID reconfigs on " + newName)

            # Do all the basic power ops
            MessWithPowerOps(vm2)
            ph.SetPhase("Basic power operations on " + newName)

            # CPU virtualization tests
            MessWithCPUVirtualization(vm2)
            ph.SetPhase("CPU Virtualization reconfigs on " + newName)

            # Monitor flag reconfigs
            MessWithReconfigMonitorUsage(vm2)
            ph.SetPhase("Monitor reconfigs on " + newName)

            # Other flag reconfigs
            MessWithReconfigMisc(vm2)
            ph.SetPhase("Other flag reconfigs on " + newName)

            # Tools reconfigs
            MessWithReconfigTools(vm2)
            ph.SetPhase("Tools reconfigs on " + newName)

            # Mess with disk sizes
            MessWithDisks(vm2, newName, datastore)
            ph.SetPhase("Disk reconfigs on " + newName)

            # Mess with SATA ctlr and devices
            MessWithSata(vm2, datastore)
            ph.SetPhase("SATA devices reconfigs on " + newName)

            # Snapshot tests
            MessWithSnapshots(newName, datastore)
            ph.SetPhase("Snapshot tests on " + newName)

            # Set and unset NPIV values
            MessWithNpivWwn(vm2)
            ph.SetPhase("Npiv messes on " + newName)

            # Check remote console functionality
            TryRemoteConsole(vm2)
            ph.SetPhase("Remote console on " + newName)

            # Do sound tests
            MessWithAudio(vm2)
            ph.SetPhase("Sound messes on " + newName)

            # Do cdrom tests
            MessWithCdroms(vm2)
            ph.SetPhase("CD-ROM messes on " + newName)

            # Do floppy tests
            MessWithFloppies(vm2)
            ph.SetPhase("Floppy messes on " + newName)

            # Do parallel port tests
            MessWithParallelPorts(vm2)
            ph.SetPhase("Parallel port messes on " + newName)

            # Do serial port tests
            MessWithSerialPorts(vm2)
            ph.SetPhase("Serial port messes on " + newName)

            # Do nic tests
            MessWithNics(vm2)
            ph.SetPhase("NIC messes on " + newName)

            # Do video card tests
            MessWithVideoCard(vm2, newName, datastore)
            ph.SetPhase("VideoCard messes on " + newName)

            # Do bios tests
            MessWithBios(vm2, datastore)
            ph.SetPhase("Bios messes on " + newName)

            # Do firmware tests
            MessWithFirmware(vm2)
            ph.SetPhase("Firmware messes on " + newName)

            # Do passthru tests
            MessWithPassthru(vm2)
            ph.SetPhase("Passthru messes on " + newName)

            # Swap placement test
            MessWithSwapPlacement(vm2, si, datastore)
            ph.SetPhase("Swap placement messes on " + newName)

            # Default power ops tests
            MessWithDefaultPowerOps(vm2)
            ph.SetPhase("Default power ops messes on " + newName)

            # Scheduled hardware upgrade tests
            MessWithScheduledHardwareUpgrade(newName, datastore)
            ph.SetPhase("Scheduled hardware upgrade messes on " + newName)

            # Extra config tests
            MessWithExtraConfig(vm2)
            ph.SetPhase("Extra config messes on " + newName)

            MessWithFork(vm2)
            ph.SetPhase("Fork messes on " + newName)

            MessWithCbrcDigesCache(vm2)
            ph.SetPhase("Cbrc messes on " + newName)

            # Guest integrity platform tests
            MessWithGuestIntegrityInfo(vm2)
            ph.SetPhase("Guest integrity platform messes on " + newName)

            # Register tests
            vmname = vm2.GetConfig().GetName()
            MessWithRegister(vm2)
            vm2 = folder.Find(vmname)
            ph.SetPhase("Register/Unregister tests on "
                        + vm2.GetConfig().GetName())

            bldType = (os.environ.get('VMBLD', '') or
                       os.environ.get('BLDTYPE', ''))
            if (about.productLineId != 'ws' and
                bldType and bldType in ['obj', 'beta']):
               MessWithBadParentDisk(vm2, datastore, newName, si)
               ph.SetPhase("Bad parent disk on " + newName)
               MessWithBadReconfig(vm2)
               ph.SetPhase("Bad reconfigure on " + newName)

            vm.Destroy(vm2)
            ph.SetPhase("Destroy vm called " + newName)

            MessWithDestroy(newName, datastore, hostName=options.host)
            ph.SetPhase("VM destroy tests on " + newName)

            MessWithEventFilterSpec()
            ph.SetPhase("Event filter spec tests")

            # Record/replay tests
            #MessWithRecordReplay(newName, datastore)
            #ph.SetPhase("Record/replay tests on " + newName)

            # XXX: Missing soft power ops from using tools vm

            # Dump a bunch of info about host config and check things
            #
            # XXX: Missing stuff from vpxa compat

            status = "PASS"

         finally:
            bigClock.finish("iteration " + str(i))
            if not IsMockup():
               Log("Hostd memory after iteration %d : %d KB" % \
                                                         (i, HostdMemSize(si)))

      except Exception as e:
         Log("Caught exception : " + str(e))
         traceback.print_exc()
         status = "FAIL"

      Log("TEST RUN COMPLETE: " + status)
      resultsArray.append(status)

   return_code = 0
   Log("Results for each iteration: ")
   for i in range(len(resultsArray)):
      if resultsArray[i] == "FAIL":
         return_code = -1
      Log("Iteration " + str(i) + ": " + resultsArray[i])

   return return_code

# Start program
if __name__ == "__main__":
    sys.exit(RunTests(sys.argv[1:]))
